@@ -12,6 +12,56 @@ setClass("CompressedList",
     )
 )
 
+setMethod("classNameForDisplay", "CompressedList",
+    function(x) sub("^Compressed", "", class(x))
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Validity.
+###
+
+.valid.CompressedList.partitioning <- function(x)
+{
+    dataLength <- NROW(x@unlistData)
+    if (nobj(x@partitioning) != dataLength)
+        "improper partitioning"
+    else NULL
+}
+.valid.CompressedList.unlistData <- function(x)
+{
+    ## FIXME: workaround to support CompressedNormalIRangesList
+    ## elementTypeX <- elementType(x)
+    elementTypeX <- elementType(new(class(x)))
+    if (!extends(class(x@unlistData), elementTypeX))
+        paste("the 'unlistData' slot must be of class", elementTypeX)
+    else NULL
+}
+.valid.CompressedList <- function(x)
+{
+    c(.valid.CompressedList.unlistData(x),
+      .valid.CompressedList.partitioning(x))
+}
+setValidity2("CompressedList", .valid.CompressedList)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### updateObject()
+###
+
+setMethod("updateObject", "CompressedList",
+    function(object, ..., verbose=FALSE)
+    {
+        ## The partitioning slot is a PartitioningByEnd object which derives
+        ## from IPosRanges so its elementType slot might need to be updated.
+        ## See "updateObject" method for IPosRanges objects for more
+        ## information.
+        object@partitioning <- updateObject(object@partitioning,
+                                            ..., verbose=verbose)
+        object
+    }
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters and setters
@@ -126,37 +176,11 @@ new_CompressedList_from_list <- function(Class, x, ..., mcols)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
-###
-
-.valid.CompressedList.partitioning <- function(x)
-{
-    dataLength <- NROW(x@unlistData)
-    if (nobj(x@partitioning) != dataLength)
-        "improper partitioning"
-    else NULL
-}
-.valid.CompressedList.unlistData <- function(x)
-{
-    ## FIXME: workaround to support CompressedNormalIRangesList
-    ## elementTypeX <- elementType(x)
-    elementTypeX <- elementType(new(class(x)))
-    if (!extends(class(x@unlistData), elementTypeX))
-        paste("the 'unlistData' slot must be of class", elementTypeX)
-    else NULL
-}
-.valid.CompressedList <- function(x)
-{
-    c(.valid.CompressedList.unlistData(x),
-      .valid.CompressedList.partitioning(x))
-}
-setValidity2("CompressedList", .valid.CompressedList)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### unlist()
 ###
 
+### Overwrite method for List objects with super fast method for CompressedList
+### objects.
 setMethod("unlist", "CompressedList",
     function(x, recursive=TRUE, use.names=TRUE)
     {
@@ -169,7 +193,10 @@ setMethod("unlist", "CompressedList",
     }
 )
 
-setAs("ANY", "CompressedList", function(from) coerceToCompressedList(from))
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Coercion.
+###
 
 coerceToCompressedList <- function(from, element.type = NULL, ...) {
   if (is(from, S4Vectors:::listClassName("Compressed", element.type)))
@@ -193,6 +220,22 @@ coerceToCompressedList <- function(from, element.type = NULL, ...) {
   to
 }
 
+setAs("ANY", "CompressedList", function(from) coerceToCompressedList(from))
+
+setListCoercions <- function(type) {
+  CompressedClass <- S4Vectors:::listClassName("Compressed", type)
+  SimpleClass <- S4Vectors:::listClassName("Simple", type)
+  Class <- S4Vectors:::listClassName("", type)
+  hasCompressedList <- CompressedClass != "CompressedList"
+  if (hasCompressedList) {
+    setAs("ANY", CompressedClass, CoercerToList(type, compress = TRUE))
+  }
+  setAs("ANY", SimpleClass, CoercerToList(type, compress = FALSE))
+  setAs("ANY", Class, CoercerToList(type, compress = hasCompressedList))
+  setAs("SimpleList", Class, CoercerToList(type, compress = FALSE))
+  setAs("list", Class, CoercerToList(type, compress = FALSE))
+}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
@@ -228,68 +271,24 @@ setMethod("extractROWS", "CompressedList",
 setMethod("getListElement", "CompressedList",
     function(x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=FALSE)
-        if (is.na(i))
+        i2 <- normalizeDoubleBracketSubscript(i, x, exact=exact,
+                                              allow.NA=TRUE,
+                                              allow.nomatch=TRUE)
+        if (is.na(i2))
             return(NULL)
         unlisted_x <- unlist(x, use.names=FALSE)
         x_partitioning <- PartitioningByEnd(x)
-        window_start <- start(x_partitioning)[i]
-        window_end <- end(x_partitioning)[i]
+        window_start <- start(x_partitioning)[i2]
+        window_end <- end(x_partitioning)[i2]
         S4Vectors:::Vector_window(unlisted_x,
                                   start=window_start,
                                   end=window_end)
     }
 )
 
-setReplaceMethod("[[", "CompressedList",
-                 function(x, i, j,..., value)
-                 {
-                     nameValue <- if (is.character(i)) i else ""
-                     i <- S4Vectors:::normargSubset2_iOnly(x, i, j, ...,
-                              .conditionPrefix="[[<-,CompressedList-method: ")
-                     if (is.null(value)) {
-                         if (i <= length(x)) # if name did not exist, could be +1
-                             x <- x[-i]
-                     } else {
-                         value <- try(as(value, elementType(x)), silent = TRUE)
-                         if (inherits(value, "try-error"))
-                             stop("cannot coerce 'value' to a ", elementType(x),
-                                  " instance")
-                         listData <- as.list(x, use.names = FALSE)
-                         listData[[i]] <- value
-                         widths <- elementNROWS(x)
-                         names(widths) <- NULL
-                         widths[i] <- NROW(value)
-                         if ((i == length(x) + 1L) &&
-                             (!is.null(names(x)) || nchar(nameValue) > 0)) {
-                             NAMES <- names(x)
-                             if (is.null(NAMES))
-                                 NAMES <- rep.int("", length(x))
-                             NAMES[i] <- nameValue
-                         } else {
-                             NAMES <- names(x)
-                         }
-                         slot(x, "unlistData", check=FALSE) <-
-                           compress_listData(listData, elementType(x))
-                         slot(x, "partitioning", check=FALSE) <-
-                           new2("PartitioningByEnd", end = cumsum(widths),
-                                NAMES = NAMES, check=FALSE)
-                         if (i > length(x))
-                           x <- S4Vectors:::rbindRowOfNAsToMetadatacols(x)
-                         x
-                     }
-                 })
-
-setReplaceMethod("$", "CompressedList",
-                 function(x, name, value) {
-                     x[[name]] <- value
-                     x
-                 })
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
+### Concatenation
 ###
 
 .bindROWS <- function(...)
@@ -297,87 +296,44 @@ setReplaceMethod("$", "CompressedList",
     args <- list(...)
     if (length(dim(args[[1L]])) >= 2L)
         return(rbind(...))
-    if (!is.factor(args[[1L]]))
-        return(c(...))
-    ans_levels <- unique(unlist(lapply(args, levels)))
-    x <- unlist(lapply(args, as.character))
-    factor(x, levels=ans_levels)
+    concatenateObjects(args[[1L]], args[-1L])
 }
 
-### Not exported.
-combine_CompressedList_objects <- function(Class, objects,
-                                           use.names=TRUE, ignore.mcols=FALSE)
+.concatenate_CompressedList_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
-    if (!isSingleString(Class))
-        stop("'Class' must be a single character string")
-    if (!extends(Class, "CompressedList"))
-        stop("'Class' must be the name of a class that extends CompressedList")
-    if (!is.list(objects))
-        stop("'objects' must be a list")
-    if (!isTRUEorFALSE(use.names))
-        stop("'use.names' must be TRUE or FALSE")
-    ### TODO: Support 'use.names=TRUE'.
-    if (use.names)
-        stop("'use.names=TRUE' is not supported yet")
-    if (!isTRUEorFALSE(ignore.mcols))
-        stop("'ignore.mcols' must be TRUE or FALSE")
+    objects <- S4Vectors:::prepare_objects_to_concatenate(x, objects)
+    all_objects <- c(list(x), objects)
 
-    if (length(objects) != 0L) {
-        ## TODO: Implement (in C) fast 'elementIsNull(objects)' in S4Vectors
-        ## that does 'sapply(objects, is.null, USE.NAMES=FALSE)', and use it
-        ## here.
-        null_idx <- which(sapply(objects, is.null, USE.NAMES=FALSE))
-        if (length(null_idx) != 0L)
-            objects <- objects[-null_idx]
-    }
-    if (length(objects) == 0L)
-        return(new(Class))
+    ## 1. Take care of the parallel slots
 
-    ## TODO: Implement (in C) fast 'elementIs(objects, class)' in S4Vectors
-    ## that does 'sapply(objects, is, class, USE.NAMES=FALSE)', and use it
-    ## here. 'elementIs(objects, "NULL")' should work and be equivalent to
-    ## 'elementIsNull(objects)'.
-    if (!all(sapply(objects, is, "CompressedList", USE.NAMES=FALSE)))
-        stop("the objects to combine must be CompressedList objects (or NULLs)")
-    objects_names <- names(objects)
-    names(objects) <- NULL  # so lapply(objects, ...) below returns an
-                            # unnamed list
+    ## Call method for Vector objects to concatenate all the parallel slots
+    ## (only "elementMetadata" in the case of CompressedList) and stick them
+    ## into 'ans'. Note that the resulting 'ans' can be an invalid object
+    ## because its "elementMetadata" slot can be longer (i.e. have more rows)
+    ## than 'ans' itself so we use 'check=FALSE' to skip validation.
+    ans <- callNextMethod(x, objects, use.names=use.names,
+                                      ignore.mcols=ignore.mcols,
+                                      check=FALSE)
 
-    ## Combine "unlistData" slots.
-    unlistData_slots <- lapply(objects, function(x) x@unlistData)
-    ans_unlistData <- do.call(.bindROWS, unlistData_slots)
+    ## 2. Take care of the non-parallel slots
 
-    ## Combine "partitioning" slots.
-    ans_breakpoints <- cumsum(unlist(lapply(objects, elementNROWS)))
+    ## Concatenate the "unlistData" slots.
+    unlistData_list <- lapply(all_objects, slot, "unlistData")
+    ans_unlistData <- do.call(.bindROWS, unlistData_list)
+
+    ## Concatenate the "partitioning" slots.
+    ans_breakpoints <- cumsum(unlist(lapply(all_objects, elementNROWS),
+                                     use.names=use.names))
     ans_partitioning <- PartitioningByEnd(ans_breakpoints)
 
-    ans <- newCompressedList0(Class, ans_unlistData, ans_partitioning)
-
-    ## Combine "mcols" slots.
-    if (!ignore.mcols) {
-        ans_mcols <- do.call(S4Vectors:::rbind_mcols, objects)
-        rownames(ans_mcols) <- NULL
-        mcols(ans) <- ans_mcols
-    }
-    ans
+    BiocGenerics:::replaceSlots(ans, unlistData=ans_unlistData,
+                                     partitioning=ans_partitioning,
+                                     check=check)
 }
 
-setMethod("c", "CompressedList",
-    function (x, ..., ignore.mcols=FALSE, recursive=FALSE)
-    {
-        if (!identical(recursive, FALSE))
-            stop("\"c\" method for CompressedList objects ",
-                 "does not support the 'recursive' argument")
-        if (missing(x)) {
-            objects <- list(...)
-            x <- objects[[1L]]
-        } else {
-            objects <- list(x, ...)
-        }
-        combine_CompressedList_objects(class(x), objects,
-                                       use.names=FALSE,
-                                       ignore.mcols=ignore.mcols)
-    }
+setMethod("concatenateObjects", "CompressedList",
+    .concatenate_CompressedList_objects
 )
 
 
@@ -410,21 +366,12 @@ lapply_CompressedList <- function(X, FUN, ...)
         return(ans)
     X_elt_start <- start(X_partitioning)
     X_elt_end <- end(X_partitioning)
-    old_validity_status <- S4Vectors:::disableValidity()
-    S4Vectors:::disableValidity(TRUE)
-    on.exit(S4Vectors:::disableValidity(old_validity_status))
     ans[non_empty_idx] <-
       lapply(non_empty_idx,
              function(i)
                  FUN(extractROWS(unlisted_X,
                                  IRanges(X_elt_start[i], X_elt_end[i])),
                      ...))
-    S4Vectors:::disableValidity(old_validity_status)
-    for (i in non_empty_idx) {
-        obj <- ans[[i]]
-        if (isS4(obj) && !isTRUE(validObject(obj, test=TRUE)))
-            stop("invalid output element of class \"", class(obj), "\"")
-    }
     ans
 }
 
@@ -437,36 +384,6 @@ setMethod("lapply", "CompressedList",
     }
 )
 
-.updateCompressedList <- function(X, listData) {
-    elementTypeX <- elementType(X)
-    if (!all(sapply(listData,
-                    function(Xi) extends(class(Xi), elementTypeX))))
-        stop("'FUN' must return elements of class ", elementTypeX)
-    if (length(listData) == 0) {
-        end <- integer(0)
-    } else {
-        end <- cumsum(unlist(lapply(listData, NROW), use.names = FALSE))
-    }
-    initialize(X,
-               unlistData = compress_listData(listData, elementTypeX),
-               partitioning = 
-               new2("PartitioningByEnd", end = end, NAMES = names(X),
-                    check=FALSE))
-}
-
-setMethod("endoapply", "CompressedList",
-          function(X, FUN, ...) {
-              .updateCompressedList(X, lapply_CompressedList(X, FUN, ...))
-          })
-
-setMethod("mendoapply", "CompressedList",
-          function(FUN, ..., MoreArgs = NULL) {
-              .updateCompressedList(list(...)[[1L]],
-                                    mapply(FUN = match.fun(FUN), ...,
-                                           MoreArgs = MoreArgs,
-                                           SIMPLIFY = FALSE))
-          })
-
 setMethod("revElements", "CompressedList",
     function(x, i)
     {
@@ -483,12 +400,3 @@ setMethod("revElements", "CompressedList",
     }
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### classNameForDisplay()
-###
-
-setMethod("classNameForDisplay", "CompressedList",
-    function(x) sub("^Compressed", "", class(x))
-)
-
