@@ -5,18 +5,15 @@
 setClass("DataFrameList", representation("VIRTUAL"),
          prototype = prototype(elementType = "DataFrame"),
          contains = "List")
+
 setClass("SimpleDataFrameList",
          contains = c("DataFrameList", "SimpleList"))
-setClass("CompressedDataFrameList",
-         prototype = prototype(unlistData = new("DataFrame")),
-         contains = c("DataFrameList", "CompressedList"))
 
 setClass("SplitDataFrameList", representation("VIRTUAL"),
          contains = "DataFrameList")
+
 setClass("SimpleSplitDataFrameList",
          contains = c("SplitDataFrameList", "SimpleDataFrameList"))
-setClass("CompressedSplitDataFrameList",
-         contains = c("SplitDataFrameList", "CompressedDataFrameList"))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessor methods.
@@ -50,16 +47,6 @@ setMethod("ncol", "SimpleSplitDataFrameList",
                         names = names(x))
           })
 
-setMethod("ncol", "CompressedSplitDataFrameList",
-          function(x)
-          {
-            if (length(x) == 0L)
-              0L
-            else
-              structure(rep.int(ncol(x@unlistData), length(x)),
-                        names = names(x))
-          })
-
 setMethod("dim", "DataFrameList",
           function(x)
           {
@@ -87,15 +74,6 @@ setMethod("colnames", "SplitDataFrameList",
             } else NULL
           })
 
-setMethod("colnames", "CompressedSplitDataFrameList",
-          function(x, do.NULL = TRUE, prefix = "col")
-          {
-            if (length(x)) {
-              nms <- colnames(x@unlistData, do.NULL = do.NULL, prefix = prefix)
-              rep(CharacterList(nms), length(x))
-            } else NULL
-          })
-
 setMethod("dimnames", "DataFrameList",
           function(x)
           {
@@ -119,21 +97,6 @@ setReplaceMethod("rownames", "SimpleDataFrameList",
                    x
                  })
 
-setReplaceMethod("rownames", "CompressedSplitDataFrameList",
-                 function(x, value)
-                 {
-                   if (is.null(value)) {
-                     rownames(x@unlistData) <- NULL
-                   } else if (is(value, "CharacterList")){
-                     if (length(x) != length(value))
-                       stop("replacement value must be the same length as x")
-                     rownames(x@unlistData) <- unlist(value, use.names=FALSE)
-                   } else {
-                     stop("replacement value must either be NULL or a CharacterList")
-                   }
-                   x
-                 })
-
 setReplaceMethod("colnames", "SimpleDataFrameList",
                  function(x, value)
                  {
@@ -154,24 +117,6 @@ setReplaceMethod("colnames", "SimpleDataFrameList",
                    x
                  })
 
-setReplaceMethod("colnames", "CompressedSplitDataFrameList",
-                 function(x, value)
-                 {
-                   if (is.null(value)) {
-                     colnames(x@unlistData) <- NULL
-                   } else if (is.character(value)) {
-                     colnames(x@unlistData) <- value
-                   } else if (is(value, "CharacterList")){
-                     if (length(x) != length(value))
-                       stop("replacement value must be the same length as x")
-                     if (length(x) > 0)
-                       colnames(x@unlistData) <- unlist(value[[1L]])
-                   } else {
-                     stop("replacement value must either be NULL or a CharacterList")
-                   }
-                   x
-                 })
-
 setReplaceMethod("dimnames", "DataFrameList",
                  function(x, value)
                  {
@@ -190,10 +135,6 @@ setMethod("columnMetadata", "SimpleSplitDataFrameList", function(x) {
   else NULL
 })
 
-setMethod("columnMetadata", "CompressedSplitDataFrameList", function(x) {
-  mcols(x@unlistData)
-})
-
 setGeneric("columnMetadata<-",
            function(x, ..., value) standardGeneric("columnMetadata<-"))
 
@@ -206,18 +147,12 @@ setReplaceMethod("columnMetadata", "SimpleSplitDataFrameList",
                    x
                  })
 
-setReplaceMethod("columnMetadata", "CompressedSplitDataFrameList",
-                 function(x, value) {
-                   mcols(x@unlistData) <- value
-                   x
-                 })
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Validity.
 ###
 
-.valid.SplitDataFrameList <- function(x) {
-  if (length(x) && !is(x, "CompressedList")) {
+.valid.SimpleSplitDataFrameList <- function(x) {
+  if (length(x)) {
     firstNames <- colnames(x[[1L]])
     l <- as.list(x, use.names = FALSE)
     if (!all(sapply(l, function(df) identical(firstNames, colnames(df)))))
@@ -231,7 +166,7 @@ setReplaceMethod("columnMetadata", "CompressedSplitDataFrameList",
   NULL
 }
 
-setValidity2("SplitDataFrameList", .valid.SplitDataFrameList)
+setValidity2("SimpleSplitDataFrameList", .valid.SimpleSplitDataFrameList)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor.
@@ -289,22 +224,6 @@ setMethod("[", "SimpleSplitDataFrameList",
             x
           })
 
-setMethod("[", "CompressedSplitDataFrameList",
-          function(x, i, j, ..., drop=TRUE)
-          {
-            if (!missing(j))
-              x@unlistData <- x@unlistData[, j, drop=FALSE]
-            if (!missing(i))
-              x <- callNextMethod(x, i)
-
-            if (((nargs() - !missing(drop)) > 2) &&
-                (ncol(x@unlistData) == 1) && (missing(drop) || drop)) {
-              x <- relist(x@unlistData[[1L]], x)
-            }
-
-            x
-          })
-
 setMethod("normalizeSingleBracketReplacementValue", "SplitDataFrameList",
     function(value, x)
     {
@@ -338,7 +257,7 @@ setReplaceMethod("[", "SplitDataFrameList",
             y <- value
         } else {
             y <- x[, j, drop=FALSE]
-            if (is.list(i) || (is(i, "List") && !is(i, "Ranges"))) {
+            if (is.list(i) || (is(i, "List") && !is(i, "IntegerRanges"))) {
                 y <- S4Vectors:::lsubset_List_by_List(y, i, value)
             } else {
                 y[i] <- value
@@ -386,9 +305,6 @@ setAs("DataFrameList", "DataFrame", function(from) {
 
 setGeneric("commonColnames", function(x) standardGeneric("commonColnames"))
 
-setMethod("commonColnames", "CompressedSplitDataFrameList",
-          function(x) colnames(unlist(x, use.names=FALSE)))
-
 setMethod("commonColnames", "SplitDataFrameList",
           function(x) colnames(head(x, 1L))[[1L]])
 
@@ -400,28 +316,16 @@ setAs("SplitDataFrameList", "DataFrame",
     }
 )
 
-setAs("ANY", "SplitDataFrameList",
-      function(from) as(from, "CompressedSplitDataFrameList"))
-
 setAs("list", "SplitDataFrameList",
       function(from) as(from, "SimpleSplitDataFrameList"))
 
 setAs("SimpleList", "SplitDataFrameList",
       function(from) as(from, "SimpleSplitDataFrameList"))
 
-setAs("DataFrame", "SplitDataFrameList",
-      function(from) as(from, "CompressedSplitDataFrameList"))
-
 setAs("ANY", "SimpleSplitDataFrameList",
       function(from) {
         new("SimpleSplitDataFrameList", as(from, "SimpleDataFrameList"))
       })
-setAs("ANY", "CompressedSplitDataFrameList",
-      function(from) {
-        coerceToCompressedList(from, "DataFrame")
-      })
-
-setListCoercions("DataFrame")
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Show
