@@ -198,8 +198,7 @@ setMethod("nobj", "H2LGrouping", function(x) length(x@high2low))
 setMethod("getListElement", "H2LGrouping",
     function(x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=TRUE)
+        i <- normalizeDoubleBracketSubscript(i, x, exact=exact)
         if (is.na(x@high2low[i]))
             c(i, x@low2high[[i]])
         else
@@ -425,15 +424,15 @@ setMethod("high2low", "ANY",
 ###
 ### A GroupingRanges object represents a "block-grouping", that is, a
 ### grouping where each group is a block of adjacent elements in the original
-### collection of objects. GroupingRanges objects support the Ranges API (e.g.
-### start/end/width) in addition to the Grouping API.
+### collection of objects. GroupingRanges objects support the IPosRanges
+### API (e.g. start/end/width) in addition to the Grouping API.
 ###
 
 setClass("GroupingRanges",
-    ## We put Ranges before Grouping so GroupingRanges objects inherit the
-    ## "show" method for Ranges objects instead of the method for Grouping
-    ## objects.
-    contains=c("Ranges", "Grouping"),
+    ## We put IPosRanges before Grouping so GroupingRanges objects inherit
+    ## the "show" method for IPosRanges objects instead of the method for
+    ## Grouping objects.
+    contains=c("IPosRanges", "Grouping"),
     representation("VIRTUAL")
 )
 
@@ -475,14 +474,38 @@ setClass("Partitioning",
     )
 )
 
+setMethod("parallelSlotNames", "Partitioning",
+    function(x) c("NAMES", callNextMethod())
+)
+
+setMethod("extractROWS", "Partitioning",
+    function(x, i)
+    {
+        i <- normalizeSingleBracketSubscript(i, x)
+        if (!isStrictlySorted(i))
+            stop(wmsg("Partitioning objects only support subsetting by a ",
+                      "strictly sorted subscript that drops empty partitions"))
+        x_nobj <- nobj(x)
+        ans <- callNextMethod()
+        if (nobj(ans) != x_nobj)
+            stop(wmsg("Partitioning objects only support subsetting by a ",
+                      "strictly sorted subscript that drops empty partitions"))
+        ans
+    }
+)
+
+setMethod("concatenateObjects", "Partitioning",
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+        stop(wmsg("Partitioning objects don't support concatenation"))
+)
+
 ### The default methods below assume that the "length + start/end/width" API
 ### is already implemented.
 
 setMethod("getListElement", "Partitioning",
     function(x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=TRUE)
+        i <- normalizeDoubleBracketSubscript(i, x, exact=exact)
         ## The purpose of the code below is to extract 'start(x)[i] - 1'
         ## (stored in 'ans_shift') and 'width(x)[i]' (stored in 'ans_len')
         ## in the fastest possible way. Looks like a convoluted way to
@@ -513,19 +536,6 @@ setMethod("names", "Partitioning", function(x) x@NAMES)
 
 setReplaceMethod("names", "Partitioning", set_IRanges_names)
 
-.valid.Partitioning <- function(x)
-{
-    if (is.null(names(x)))
-        return(NULL)
-    if (!is.character(names(x)))
-        return("the names must be a character vector or NULL")
-    if (length(names(x)) != length(x))
-        return("number of names and number of elements differ")
-    NULL
-}
-
-setValidity2("Partitioning", .valid.Partitioning)
-
 setMethod("NSBS", "Partitioning",
           function(i, x, exact=TRUE, strict.upper.bound=TRUE, allow.NAs=FALSE)
           {
@@ -549,6 +559,10 @@ setClass("PartitioningByEnd",
     )
 )
 
+setMethod("parallelSlotNames", "PartitioningByEnd",
+    function(x) c("end", callNextMethod())
+)
+
 setMethod("end", "PartitioningByEnd", function(x) x@end)
 
 ### Overwrite method for Ranges objects with optimized method for
@@ -679,13 +693,13 @@ PartitioningByEnd <- function(x=integer(0), NG=NULL, names=NULL)
                               check=FALSE)
 }
 
-setAs("Ranges", "PartitioningByEnd",
+setAs("IntegerRanges", "PartitioningByEnd",
     function(from)
     {
         ans <- PartitioningByEnd(end(from), names=names(from))
         if (!identical(start(ans), start(from)))
-            stop(wmsg("the Ranges object to coerce does not represent ",
-                      "a partitioning"))
+            stop(wmsg("the IntegerRanges object to coerce does not ",
+                      "represent a partitioning"))
         ans
     }
 )
@@ -708,8 +722,16 @@ setClass("PartitioningByWidth",
     )
 )
 
+setMethod("parallelSlotNames", "PartitioningByWidth",
+    function(x) c("width", callNextMethod())
+)
+
 setMethod("width", "PartitioningByWidth", function(x) x@width)
 
+### Overwrite method for Ranges objects with optimized method for
+### PartitioningByWidth objects.
+setMethod("length", "PartitioningByWidth", function(x) length(width(x)))
+
 setMethod("end", "PartitioningByWidth", function(x) cumsum(width(x)))
 
 setMethod("start", "PartitioningByWidth",
@@ -800,13 +822,13 @@ PartitioningByWidth <- function(x=integer(0), NG=NULL, names=NULL)
                                 check=FALSE)
 }
 
-setAs("Ranges", "PartitioningByWidth",
+setAs("IntegerRanges", "PartitioningByWidth",
     function(from)
     {
         ans <- PartitioningByWidth(width(from), names(from))
         if (!identical(start(ans), start(from)))
-            stop(wmsg("the Ranges object to coerce does not represent ",
-                      "a partitioning"))
+            stop(wmsg("the IntegerRanges object to coerce does not ",
+                      "represent a partitioning"))
         ans
     }
 )
@@ -882,12 +904,12 @@ setMethod("show", "PartitioningMap",
 ### Partitioning object).
 ### TODO: Add a "findOverlaps" method for Partitioning,Partitioning in the
 ### findOverlaps-methods.R file that calls this.
-findOverlaps_Ranges_Partitioning <- function(query, subject,
-                                             hit.empty.query.ranges=FALSE,
-                                             hit.empty.subject.ranges=FALSE)
+findOverlaps_IntegerRanges_Partitioning <-
+    function(query, subject, hit.empty.query.ranges=FALSE,
+                             hit.empty.subject.ranges=FALSE)
 {
-    if (!is(query, "Ranges"))
-        stop(wmsg("'query' must be a Ranges object"))
+    if (!is(query, "IntegerRanges"))
+        stop(wmsg("'query' must be an IntegerRanges object"))
     if (!is(subject, "Partitioning"))
         stop(wmsg("'subject' must be a Partitioning object"))
     if (!isTRUEorFALSE(hit.empty.query.ranges) ||
