@@ -65,10 +65,10 @@ solveUserSEW0 <- function(start=NULL, end=NULL, width=NULL)
 
 IRanges <- function(start=NULL, end=NULL, width=NULL, names=NULL)
 {
-    if (is(start, "Ranges")) {
+    if (is(start, "IntegerRanges")) {
         if (!is.null(end) || !is.null(width))
             stop("'end' and 'width' must be NULLs ",
-                 "when 'start' is a Ranges object")
+                 "when 'start' is an IntegerRanges object")
         ans <- new2("IRanges", start=start(start), width=width(start),
                     NAMES=names, check=FALSE)
         return(ans)
