@@ -9,7 +9,7 @@
 
 ### We deliberately do NOT extend IRanges.
 setClass("NCList",
-    contains="Ranges",
+    contains="IPosRanges",
     representation(
         nclist="integer",
         ranges="IRanges"
@@ -113,8 +113,8 @@ setMethod("width", "NCList", function(x) width(x@ranges))
 
 NCList <- function(x, circle.length=NA_integer_)
 {
-    if (!is(x, "Ranges"))
-        stop("'x' must be a Ranges object")
+    if (!is(x, "IntegerRanges"))
+        stop("'x' must be an IntegerRanges object")
     if (!is(x, "IRanges"))
         x <- as(x, "IRanges")
     ans_mcols <- mcols(x)
@@ -128,8 +128,6 @@ NCList <- function(x, circle.length=NA_integer_)
                    check=FALSE)
 }
 
-setAs("Ranges", "NCList", function(from) NCList(from))
-
 ### NOT exported.
 print_NCList <- function(x)
 {
@@ -140,6 +138,30 @@ print_NCList <- function(x)
     invisible(NULL)
 }
 
+setAs("IntegerRanges", "NCList", function(from) NCList(from))
+
+### Inefficient because it rebuilds a new NCList object from scratch for the
+### selected ranges. Supported for completeness only!
+setMethod("extractROWS", c("NCList", "ANY"),
+    function(x, i)
+    {
+        x_class <- class(x)
+        x <- as(x, "IRanges")
+        as(callGeneric(), x_class)
+    }
+)
+
+### Inefficient because it rebuilds a new NCList object from scratch for the
+### concatenated ranges. Supported for completeness only!
+setMethod("concatenateObjects", "NCList",
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+    {
+        x_class <- class(x)
+        x <- as(x, "IRanges")
+        as(callGeneric(), x_class)
+    }
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### findOverlaps_NCList()
@@ -152,8 +174,8 @@ findOverlaps_NCList <- function(query, subject,
              select=c("all", "first", "last", "arbitrary", "count"),
              circle.length=NA_integer_)
 {
-    if (!(is(query, "Ranges") && is(subject, "Ranges")))
-        stop("'query' and 'subject' must be Ranges objects")
+    if (!(is(query, "IntegerRanges") && is(subject, "IntegerRanges")))
+        stop("'query' and 'subject' must be IntegerRanges objects")
 
     if (!isSingleNumber(maxgap))
         stop("'maxgap' must be a single integer")
@@ -198,7 +220,7 @@ findOverlaps_NCList <- function(query, subject,
 ###
 
 setClass("NCLists",
-    contains="RangesList",
+    contains="IntegerRangesList",
     representation(
         nclists="list",
         rglist="CompressedIRangesList"
@@ -241,8 +263,7 @@ setMethod("elementNROWS", "NCLists", function(x) elementNROWS(x@rglist))
 setMethod("getListElement", "NCLists",
     function (x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=TRUE)
+        i <- normalizeDoubleBracketSubscript(i, x, exact=exact)
         new2("NCList", nclist=x@nclists[[i]], ranges=x@rglist[[i]],
                        check=FALSE)
     }
@@ -258,7 +279,7 @@ setAs("NCLists", "IRangesList",
 .extract_groups_from_RangesList <- function(x)
 {
     x_partitioning <- PartitioningByEnd(x)
-    relist(as.integer(x_partitioning) - 1L, x_partitioning)
+    relist(unlist_as_integer(x_partitioning) - 1L, x_partitioning)
 }
 
 .nclists <- function(x, x_groups)
@@ -271,8 +292,8 @@ setAs("NCLists", "IRangesList",
 ### NCLists constructor.
 NCLists <- function(x, circle.length=NA_integer_)
 {
-    if (!is(x, "RangesList"))
-        stop("'x' must be a RangesList object")
+    if (!is(x, "IntegerRangesList"))
+        stop("'x' must be an IntegerRangesList object")
     if (!is(x, "CompressedIRangesList"))
         x <- as(x, "CompressedIRangesList")
     ans_mcols <- mcols(x)
@@ -293,7 +314,7 @@ NCLists <- function(x, circle.length=NA_integer_)
                     check=FALSE)
 }
 
-setAs("RangesList", "NCLists", function(from) NCLists(from))
+setAs("IntegerRangesList", "NCLists", function(from) NCLists(from))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -311,8 +332,8 @@ NCList_find_overlaps_in_groups <- function(
              select=c("all", "first", "last", "arbitrary", "count"),
              circle.length)
 {
-    if (!(is(q, "Ranges") && is(s, "Ranges")))
-        stop("'q' and 's' must be Ranges object")
+    if (!(is(q, "IntegerRanges") && is(s, "IntegerRanges")))
+        stop("'q' and 's' must be IntegerRanges object")
     if (!is(q_groups, "CompressedIntegerList"))
         stop("'q_groups' must be a CompressedIntegerList object")
     if (!is(s_groups, "CompressedIntegerList"))
@@ -395,8 +416,8 @@ findOverlaps_NCLists <- function(query, subject,
              select=c("all", "first", "last", "arbitrary", "count"),
              circle.length=NA_integer_)
 {
-    if (!(is(query, "RangesList") && is(subject, "RangesList")))
-        stop("'query' and 'subject' must be RangesList objects")
+    if (!(is(query, "IntegerRangesList") && is(subject, "IntegerRangesList")))
+        stop("'query' and 'subject' must be IntegerRangesList objects")
     type <- match.arg(type)
     select <- match.arg(select)
     circle.length <- .normarg_circle.length2(circle.length,
