@@ -8,10 +8,10 @@
 ## 1) Efficiency when data is large (i.e. apply by chromosome)
 ## 2) Convenience when data is not so large (i.e. unrolling the data)
 
-## The ranges are stored in a RangesList, while the data is stored in
-## a SplitDataFrameList. The RangesList is uncompressed, because
-## users will likely want to apply over each Ranges separately, as
-## they are usually in separate spaces. Also, it is difficult to
+## The ranges are stored in a IntegerRangesList, while the data is stored
+## in a SplitDataFrameList. The IntegerRangesList is uncompressed, because
+## users will likely want to apply over each IntegerRanges separately,
+## as they are usually in separate spaces. Also, it is difficult to
 ## compress RangesLists, as lists containing Views or NCLists
 ## are uncompressible. The SplitDataFrameList should be compressed,
 ## because it's cheap to create from a split factor and, more
@@ -21,8 +21,9 @@
 ## performance penalty when applying over the RangedData.
 
 setClass("RangedData", contains = c("DataTable", "List"),
-         representation(ranges = "RangesList", values = "SplitDataFrameList"),
-         prototype = prototype(ranges = new("SimpleRangesList"),
+         representation(ranges = "IntegerRangesList",
+                        values = "SplitDataFrameList"),
+         prototype = prototype(ranges = new("SimpleIRangesList"),
                                values = new("CompressedSplitDataFrameList")))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -58,13 +59,13 @@ setMethod("ranges", "RangedData",
 
 setReplaceMethod("ranges", "RangedData",
                  function(x, value) {
-                   if (extends(class(value), "RangesList")) {
+                   if (extends(class(value), "IntegerRangesList")) {
                      if (!identical(lapply(ranges(x), names), lapply(value, names)))
                        stop("'value' must have same length and names as current 'ranges'")
                    } else if (extends(class(value), "IRanges")) {
                      value <- split(value, space(x))
                    } else {
-                     stop("'value' must extend class RangesList or IRanges")
+                     stop("'value' must extend class IntegerRangesList or IRanges")
                    }
                    x@ranges <- value
                    x
@@ -239,14 +240,12 @@ RangedData <- function(ranges = IRanges(), ..., space = NULL, universe = NULL)
   hasDots <- (((nargs() - !missing(space)) - !missing(universe)) > 1)
   if (!is.null(universe)) {
      msg <- wmsg("The 'universe' argument of the RangedData() ",
-                 "constructor function is deprecated.")
-    .Deprecated(msg=msg)
-    if (!isSingleString(universe))
-      stop("'universe' must be a single string")
+                 "constructor function is defunct.")
+    .Defunct(msg=msg)
   }
-  if (is(ranges, "RangesList")) {
+  if (is(ranges, "IntegerRangesList") && !is(ranges, "IntegerRanges")) {
     if (!is.null(space))
-      warning("since 'class(ranges)' extends RangesList, 'space' argument is ignored")
+      warning("since 'class(ranges)' extends IntegerRangesList, 'space' argument is ignored")
     if (is.null(names(ranges)))
       names(ranges) <- as.character(seq_len(length(ranges)))
     space <-
@@ -255,11 +254,11 @@ RangedData <- function(ranges = IRanges(), ..., space = NULL, universe = NULL)
     N <- sum(elementNROWS(ranges))
     NAMES <- unlist(lapply(ranges, names), use.names=FALSE)
   } else {
-    if (!is(ranges, "Ranges")) {
+    if (!is(ranges, "IntegerRanges")) {
       coerced <- try(as(ranges, "RangedData"), silent=TRUE)
       if (is(coerced, "RangedData"))
         return(coerced)
-      stop("'ranges' must be a Ranges or directly coercible to RangedData")
+      stop("'ranges' must be an IntegerRanges or directly coercible to RangedData")
     }
     N <- length(ranges)
     NAMES <- names(ranges)
@@ -391,11 +390,6 @@ setReplaceMethod("[[", "RangedData",
                    x
                  })
 
-setReplaceMethod("$", "RangedData", function(x, name, value) {
-  x[[name]] <- value
-  x
-})
-
 ### Supported index types: numeric, logical, character, NULL and missing.
 ## Two index modes:
 ## - list style ([i]):  subsets by range space (e.g. chromosome)
@@ -444,9 +438,9 @@ setMethod("[", "RangedData",
                 values <- values[, j, drop=FALSE]
               }
               if (!missing(i)) {
-                if (is(i, "RangesList"))
+                if (is(i, "IntegerRangesList"))
                   stop("subsetting a RangedData object ",
-                       "by a RangesList subscript is not supported")
+                       "by an IntegerRangesList subscript is not supported")
                 if (is(i, "LogicalList")) {
                   x_eltNROWS <- elementNROWS(ranges(x))
                   whichRep <- which(x_eltNROWS != elementNROWS(i))
@@ -569,17 +563,18 @@ setMethod("rbind", "RangedData", function(..., deparse.level=1) {
 ### Coercion
 ###
 
-### The 2 functions, as.data.frame.RangesList() and
+### The 2 functions, as.data.frame.IntegerRangesList() and
 ### as.data.frame.DataFrameList() are needed for as.data.frame.RangedData().
 ###
 ### A new as.data.frame,List method was implemented in BioC 2.15 and
 ### is now used by all List classes. Because the RangedData class is being 
 ### phased out, we want to retain the old behavior. In order to do that 
 ### we have to keep these 2 helpers because as.data.frame.RangedData() 
-### uses old methods from both RangesList and DataFrameList.
+### uses old methods from both IntegerRangesList and DataFrameList.
 ###
 ### These helpers are not exported.
-.as.data.frame.RangesList <- function(x, row.names=NULL, optional=FALSE, ...)
+.as.data.frame.IntegerRangesList <- function(x, row.names=NULL, optional=FALSE,
+                                             ...)
 {
     if (!(is.null(row.names) || is.character(row.names)))
         stop("'row.names'  must be NULL or a character vector")
@@ -618,7 +613,7 @@ setMethod("rbind", "RangedData", function(..., deparse.level=1) {
         stop("'row.names'  must be NULL or a character vector")
     if (!missing(optional) || length(list(...)))
         warning("'optional' and arguments in '...' ignored")
-    data.frame(.as.data.frame.RangesList(ranges(x)),
+    data.frame(.as.data.frame.IntegerRangesList(ranges(x)),
                .as.data.frame.DataFrameList(values(x))[-1L],
                row.names = row.names,
                stringsAsFactors = FALSE)
@@ -670,7 +665,8 @@ setAs("RleViewsList", "RangedData", function(from) {
                           keep.all.ranges = TRUE)
 ### FIXME: do we want to insert NAs for out of bounds views?
   score <- subject[from_ranges]
-  score_part <- as(lapply(width(from_ranges), PartitioningByWidth), "RangesList")
+  score_part <- as(lapply(width(from_ranges), PartitioningByWidth),
+                   "IntegerRangesList")
   score_ranges <- ranges(score)
   ol <- findOverlaps(score_ranges, score_part)
   offind <- as(lapply(ol, subjectHits), "IntegerList")
@@ -686,18 +682,18 @@ setAs("RleViewsList", "RangedData", function(from) {
              view = unlist(viewNames, use.names = FALSE)[subjectHits(ol)])
 })
 
-setAs("Ranges", "RangedData",
+setAs("IntegerRanges", "RangedData",
       function(from)
       {
         RangedData(from)
       })
 
-setAs("RangesList", "RangedData",
+setAs("IntegerRangesList", "RangedData",
     function(from)
     {
         from_names <- names(from)
         if (is.null(from_names) || anyDuplicated(from_names))
-            stop("cannot coerce a RangesList object with no names ",
+            stop("cannot coerce a IntegerRangesList object with no names ",
                  "or duplicated names to a RangedData object")
         unlisted_from <- unlist(from, use.names=FALSE)
         unlisted_values <- mcols(unlisted_from)
@@ -725,6 +721,8 @@ setAs("RangesList", "RangedData",
     }
 )
 
+setAs("list", "RangedData", function(from) do.call(c, unname(from)))
+
 .fromRangedDataToCompressedIRangesList <- function(from)
 {
     ans <- ranges(from)
@@ -738,7 +736,6 @@ setAs("RangedData", "CompressedIRangesList",
     .fromRangedDataToCompressedIRangesList
 )
 setAs("RangedData", "IRangesList", .fromRangedDataToCompressedIRangesList)
-setAs("RangedData", "RangesList", .fromRangedDataToCompressedIRangesList)
 
 setMethod("as.env", "RangedData", function(x, enclos = parent.frame(2)) {
   env <- callNextMethod(x, enclos)
