@@ -7,7 +7,7 @@
 ###
 
 setClass("Views",
-    contains="List",
+    contains="IntegerRanges",
     representation(
         "VIRTUAL",
         subject="Vector",
@@ -16,6 +16,17 @@ setClass("Views",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### parallelSlotNames()
+###
+
+### Combine the new parallel slots with those of the parent class. Make sure
+### to put the new parallel slots *first*.
+setMethod("parallelSlotNames", "Views",
+    function(x) c("ranges", callNextMethod())
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessor methods.
 ###
@@ -41,10 +52,7 @@ setReplaceMethod("ranges", "Views",
 ### Methods derived from the IRanges interface.
 ###
 
-setMethod("length", "Views", function(x) length(ranges(x)))
-
 setMethod("start", "Views", function(x, ...) start(ranges(x)))
-setMethod("end", "Views", function(x, ...) end(ranges(x)))
 setMethod("width", "Views", function(x) width(ranges(x)))
 setMethod("names", "Views", function(x) names(ranges(x)))
 
@@ -80,32 +88,6 @@ setReplaceMethod("names", "Views",
     }
 )
 
-setMethod("extractROWS", "Views",
-    function(x, i)
-    {
-        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
-        x@ranges <- extractROWS(ranges(x), i)
-        mcols(x) <- extractROWS(mcols(x), i)
-        x
-    }
-)
-
-setMethod("elementNROWS", "Views", function(x) width(x))
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
-###
-
-.valid.Views.width <- function(x)
-{
-    if (length(width(x)) != 0L && min(width(x)) <= 0L)
-        return("null widths are not allowed")
-    NULL
-}
-
-setValidity2("Views", .valid.Views.width)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor
@@ -118,10 +100,10 @@ setValidity2("Views", .valid.Views.width)
 new_Views <- function(subject, start=NULL, end=NULL, width=NULL, names=NULL,
                       Class=NULL)
 {
-    if (is(start, "Ranges")) {
+    if (is(start, "IntegerRanges")) {
         if (!is.null(end) || !is.null(width))
             stop(wmsg("'end' and 'width' must be NULLs when ",
-                      "'start' is a Ranges object"))
+                      "'start' is an IntegerRanges object"))
         ans_ranges <- start
         if (class(ans_ranges) != "IRanges")
             ans_ranges <- as(ans_ranges, "IRanges")
@@ -151,6 +133,24 @@ setGeneric("Views", signature="subject",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### unlist()
+###
+
+### Overwrite method for List objects with optimized method for Views objects.
+setMethod("unlist", "Views",
+    function(x, recursive=TRUE, use.names=TRUE)
+    {
+        if (!isTRUEorFALSE(use.names))
+            stop("'use.names' must be TRUE or FALSE")
+        unlisted_x <- subject(x)[ranges(x)]
+        if (use.names)
+            unlisted_x <- S4Vectors:::set_unlisted_names(unlisted_x, x)
+        unlisted_x
+    }
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion.
 ###
@@ -160,7 +160,7 @@ setAs("Vector", "Views",
     function(from) Views(from, start=1L, width=length(from))
 )
 
-setAs("Views", "Ranges", function(from) ranges(from))
+setAs("Views", "IntegerRanges", function(from) ranges(from))
 setAs("Views", "IRanges", function(from) ranges(from))
 
 ### Unfortunately, even if we've already defined the IRanges->NormalIRanges
@@ -186,12 +186,13 @@ setMethod("as.matrix", "Views", function(x, rev = FALSE, max.width = NA) {
   v <- extractROWS(subject(x), x_ranges)[ord]
   v_fill <- rep.int(NA, max.width * length(x))
   part <- PartitioningByWidth(rep(max.width, length(x)))
-  i <- as.integer(IRanges(start(part), width = width(x_ranges)))
+  i <- unlist_as_integer(IRanges(start(part), width = width(x_ranges)))
   v_fill[i] <- as.vector(v)
   matrix(v_fill, ncol = max.width, byrow = TRUE,
          dimnames = list(names(x), NULL))
 })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Extracting a view.
 ###
@@ -199,8 +200,7 @@ setMethod("as.matrix", "Views", function(x, rev = FALSE, max.width = NA) {
 setMethod("getListElement", "Views",
     function(x, i, exact=TRUE)
     {
-        i <- normalizeDoubleBracketSubscript(i, x, exact=exact,
-                                             error.if.nomatch=TRUE)
+        i <- normalizeDoubleBracketSubscript(i, x, exact=exact)
         start <- start(x)[i]
         end <- end(x)[i]
         if (start < 1L || end > length(subject(x)))
@@ -211,44 +211,31 @@ setMethod("getListElement", "Views",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining Views objects.
+### Concatenation
 ###
 
-setMethod("c", "Views",
-    function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
-    {
-        if (!identical(recursive, FALSE))
-            stop("\"c\" method for Views objects ",
-                 "does not support the 'recursive' argument")
-        if (!isTRUEorFALSE(ignore.mcols))
-            stop("'ignore.mcols' must be TRUE or FALSE")
-        if (missing(x)) {
-            args <- unname(list(...))
-            x <- args[[1L]]
-        } else {
-            args <- unname(list(x, ...))
-        }
-        if (length(args) == 1L)
-            return(x)
-        arg_is_null <- sapply(args, is.null)
-        if (any(arg_is_null))
-            args[arg_is_null] <- NULL  # remove NULL elements by setting them to NULL!
-        if (!all(sapply(args, is, class(x))))
-            stop("all arguments in '...' must be ", class(x), " objects (or NULLs)")
-        ok <- sapply(args, function(arg)
-                     isTRUE(all.equal(subject(arg), subject(x))))
-        if (!all(ok))
-            stop("all Views objects to combine must have the same subject")
-        x@ranges <- do.call(c, lapply(args, ranges))
-        if (ignore.mcols) {
-            mcols(x) <- NULL
-        } else  {
-            mcols(x) <- do.call(S4Vectors:::rbind_mcols, args)
-        }
-        validObject(x)
-        x
-    }
-)
+.check_that_Views_objects_are_concatenable <- function(x, objects)
+{
+    ok <- vapply(
+        objects,
+        function(object) isTRUE(all.equal(subject(object), subject(x))),
+        logical(1),
+        USE.NAMES=FALSE
+    )
+    if (!all(ok))
+        stop(wmsg("the Views objects to concatenate ",
+                  "must have the same subject"))
+}
+
+.concatenate_Views_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+{
+    objects <- S4Vectors:::prepare_objects_to_concatenate(x, objects)
+    .check_that_Views_objects_are_concatenable(x, objects)
+    callNextMethod()
+}
+
+setMethod("concatenateObjects", "Views", .concatenate_Views_objects)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -301,6 +288,7 @@ successiveViews <- function(subject, width, gapwidth=0, from=1)
     Views(subject, ranges)
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "slidingViews" function.
 ###
@@ -382,6 +370,7 @@ setMethod("which.min", "Views", function(x) {
   viewWhichMins(x, na.rm = TRUE)
 })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Show
 ###
