@@ -2,16 +2,16 @@
 ### ViewsList objects
 ### -------------------------------------------------------------------------
 
+
 setClass("ViewsList",
-    contains="List",
+    contains="IntegerRangesList",
     representation("VIRTUAL"),
     prototype(elementType="Views")
 )
 
 setClass("SimpleViewsList",
     contains=c("ViewsList", "SimpleList"),
-    representation("VIRTUAL"),
-    prototype(elementType="Views")
+    representation("VIRTUAL")
 )
 
 
@@ -33,23 +33,14 @@ setMethod("width", "SimpleViewsList", function(x) width(ranges(x)))
 setMethod("universe", "ViewsList",
           function(x)
           {
-            .Deprecated(msg="The universe() getter is deprecated.")
-            ### FIXME: for compatibility with older versions, eventually emit warning
-            if (is.null(metadata(x)) || is.character(metadata(x)))
-              metadata(x)
-            else
-              metadata(x)$universe
+            .Defunct(msg="The universe() getter is defunct.")
           })
 
 ### TODO: Why not define this at the List level? Or even at the Vector level?
 setReplaceMethod("universe", "ViewsList",
                  function(x, value)
                  {
-                   .Deprecated(msg="The universe() setter is deprecated.")
-                   if (!is.null(value) && !isSingleString(value))
-                     stop("'value' must be a single string or NULL")
-                   metadata(x)$universe <- value
-                   x
+                   .Defunct(msg="The universe() setter is defunct.")
                  })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -61,7 +52,9 @@ setMethod("as.matrix", "ViewsList",
           {
             if (!isTRUEorFALSE(use.names))
               stop("use.names must be TRUE or FALSE")
-            rev <- normargAtomicList1(rev, LogicalList, length(x))
+            if (!is(rev, "List"))
+              rev <- as(rev, "List")
+            rev <- S4Vectors:::VH_recycle(rev, x, "rev", "x")
             max_width <- max(max(width(restrict(ranges(x), start = 1L))))
             m <- do.call(rbind, mapply(as.matrix, x, rev,
                                        IntegerList(max_width),
@@ -71,7 +64,7 @@ setMethod("as.matrix", "ViewsList",
               nms <- rep(nms, elementNROWS(x))
               rownms <- rownames(m)
               if (is.null(rownms))
-                rownms <- as.integer(IRanges(1L, width = elementNROWS(x)))
+                rownms <- unlist_as_integer(IRanges(1L, width=elementNROWS(x)))
               rownames(m) <- paste(nms, rownms, sep = ".")
             }
             m
