@@ -4,7 +4,7 @@
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### .IRanges.coverage() and .CompressedIRangesList.coverage()
+### .coverage_IRanges() and coverage_CompressedIRangesList()
 ###
 ### These 2 internal helpers are the workhorses behind most "coverage"
 ### methods. All the hard work is almost entirely performed at the C level.
@@ -21,10 +21,9 @@
 }
 
 ### Returns an Rle object.
-.IRanges.coverage <- function(x,
-                              shift=0L, width=NULL,
-                              weight=1L, circle.length=NA,
-                              method=c("auto", "sort", "hash"))
+.coverage_IRanges <- function(x, shift=0L, width=NULL,
+                                 weight=1L, circle.length=NA,
+                                 method=c("auto", "sort", "hash"))
 {
     ## Check 'x'.
     if (!is(x, "IRanges"))
@@ -68,14 +67,14 @@
     .fold_and_truncate_coverage(ans, circle.length, width)
 }
 
-### Returns an ordinary list.
-.normarg_shift_or_weight <- function(arg, arg.label)
+### Return an ordinary list.
+.normarg_shift_or_weight_list <- function(arg, argname)
 {
     if (!is.list(arg)) {
         if (!(is.numeric(arg) ||
               (is(arg, "Rle") && is.numeric(runValue(arg))) ||
               is(arg, "List")))
-            stop("'", arg.label, "' must be a numeric vector ",
+            stop("'", argname, "' must be a numeric vector ",
                  "or a list-like object")
         arg <- as.list(arg)
     }
@@ -87,24 +86,24 @@
     arg
 }
 
-.check_arg_names <- function(arg, arg.label, x_names, x_names.label)
+.check_arg_names <- function(arg, argname, x_names, x_names.label)
 {
     arg_names <- names(arg)
     if (!(is.null(arg_names) || identical(arg_names, x_names)))
-        stop("when '", arg.label, "' has names, ",
+        stop("when '", argname, "' has names, ",
              "they must be identical to ", x_names.label)
 }
 
 ## Some packages like easyRNASeq or TEQC pass 'width' as a named list-like
 ## object where each list element is a single number, an NA, or a NULL, when
-## calling coverage() on a RangesList or RangedData object. They do so because,
-## for whatever reason, we've been supporting this for a while, and also
-## because, in the case of the method for RangedData objects, the arg default
-## for 'width' used to be such a list (a named list of NULLs in that case).
-## However, it never really made sense to support a named list-like object for
-## 'width', and it makes even less sense now that the signature of the method
-## for RangedData objects has been modified (as of BioC 2.13) to use the same
-## arg defaults as the coverage() generic and all other methods.
+## calling coverage() on an IntegerRangesList or RangedData object. They do so
+## because, for whatever reason, we've been supporting this for a while, and
+## also because, in the case of the method for RangedData objects, the arg
+## default for 'width' used to be such a list (a named list of NULLs in that
+## case). However, it never really made sense to support a named list-like
+## object for 'width', and it makes even less sense now that the signature of
+## the method for RangedData objects has been modified (as of BioC 2.13) to use
+## the same arg defaults as the coverage() generic and all other methods.
 ## TODO: Deprecate support for this. Preferred 'width' form: NULL or an integer
 ## vector. An that's it.
 .unlist_width <- function(width, x_names, x_names.label)
@@ -120,12 +119,13 @@
     setNames(unlist(width, use.names=FALSE), x_names)
 }
 
-### Returns a SimpleRleList object of the length of 'x'.
-.CompressedIRangesList.coverage <- function(x,
-                                            shift=0L, width=NULL,
-                                            weight=1L, circle.length=NA,
-                                            method=c("auto", "sort", "hash"),
-                                            x_names.label="'x' names")
+### NOT exported but used in the GenomicRanges package.
+### Return a SimpleRleList object of the length of 'x'.
+coverage_CompressedIRangesList <- function(x,
+                                           shift=0L, width=NULL,
+                                           weight=1L, circle.length=NA,
+                                           method=c("auto", "sort", "hash"),
+                                           x_names.label="'x' names")
 {
     ## Check 'x'.
     if (!is(x, "CompressedIRangesList"))
@@ -133,7 +133,7 @@
     x_names <- names(x)
 
     ## Check and normalize 'shift'.
-    shift <- .normarg_shift_or_weight(shift, "shift")
+    shift <- .normarg_shift_or_weight_list(shift, "shift")
     .check_arg_names(shift, "shift", x_names, x_names.label)
 
     ## Check and normalize 'width'.
@@ -155,7 +155,7 @@
     }
 
     ## Check and normalize 'weight'.
-    weight <- .normarg_shift_or_weight(weight, "weight")
+    weight <- .normarg_shift_or_weight_list(weight, "weight")
     .check_arg_names(weight, "weight", x_names, x_names.label)
 
     ## Check and normalize 'circle.length'.
@@ -215,24 +215,48 @@ setGeneric("coverage", signature="x",
         standardGeneric("coverage")
 )
 
-setMethod("coverage", "Ranges",
+### NOT exported but used in the GenomicRanges package.
+replace_with_mcol_if_single_string <- function(arg, x)
+{
+    if (!isSingleString(arg))
+        return(arg)
+    x_mcols <- mcols(x)
+    j <- which(colnames(x_mcols) == arg)
+    if (length(j) == 0L)
+        stop(wmsg("'mcols(x)' has no \"", arg, "\" column"))
+    if (length(j) > 1L)
+        stop(wmsg("'mcols(x)' has more than one \"", arg, "\" column"))
+    x_mcols[[j]]
+}
+
+setMethod("coverage", "IntegerRanges",
     function(x, shift=0L, width=NULL, weight=1L,
                 method=c("auto", "sort", "hash"))
     {
-        if (isSingleString(weight)) {
-            x_mcols <- mcols(x)
-            if (!is(x_mcols, "DataTable")
-             || sum(colnames(x_mcols) == weight) != 1L)
-                stop("'mcols(x)' has 0 or more than 1 \"",
-                     weight, "\" columns")
-            weight <- x_mcols[[weight]]
-        }
-        .IRanges.coverage(as(x, "IRanges"),
+        shift <- replace_with_mcol_if_single_string(shift, x)
+        weight <- replace_with_mcol_if_single_string(weight, x)
+        .coverage_IRanges(as(x, "IRanges"),
                           shift=shift, width=width, weight=weight,
                           method=method)
     }
 )
 
+### Overwrite above method with optimized method for IPos objects.
+setMethod("coverage", "IPos",
+    function(x, shift=0L, width=NULL, weight=1L,
+                method=c("auto", "sort", "hash"))
+    {
+        CAN_ONLY_ETC <- c(" can only be a single number when ",
+                          "calling coverage() on an IPos object")
+        if (!isSingleNumber(shift))
+            stop(wmsg("'shift'", CAN_ONLY_ETC))
+        if (!isSingleNumber(weight))
+            stop(wmsg("'weight'", CAN_ONLY_ETC))
+        x <- stitch_IPos(x)
+        callGeneric()
+    }
+)
+
 setMethod("coverage", "Views",
     function(x, shift=0L, width=NULL, weight=1L,
                 method=c("auto", "sort", "hash"))
@@ -247,7 +271,7 @@ setMethod("coverage", "Views",
     }
 )
 
-setMethod("coverage", "RangesList",
+setMethod("coverage", "IntegerRangesList",
     function(x, shift=0L, width=NULL, weight=1L,
                 method=c("auto", "sort", "hash"))
     {
@@ -271,15 +295,15 @@ setMethod("coverage", "RangesList",
                      "is not a valid metadata column name of 'x'")
             weight <- x_mcols[[weight]]
         }
-        .CompressedIRangesList.coverage(as(x, "CompressedIRangesList"),
-                                        shift=shift, width=width,
-                                        weight=weight,
-                                        method=method)
+        coverage_CompressedIRangesList(as(x, "CompressedIRangesList"),
+                                       shift=shift, width=width,
+                                       weight=weight,
+                                       method=method)
     }
 )
 
-.coverage_RangedData_deprecation_msg <- c(
-    "The \"coverage\" method for RangedData objects is deprecated ",
+.coverage_RangedData_defunct_msg <- c(
+    "The \"coverage\" method for RangedData objects is defunct ",
     "and won't be replaced. Please migrate your code to use GRanges or ",
     "GRangesList objects instead. RangedData objects will be deprecated ",
     "soon (their use has been discouraged since BioC 2.12, that is, since ",
@@ -290,20 +314,7 @@ setMethod("coverage", "RangedData",
     function(x, shift=0L, width=NULL, weight=1L,
                 method=c("auto", "sort", "hash"))
     {
-        .Deprecated(msg=wmsg(.coverage_RangedData_deprecation_msg))
-        x_ranges <- ranges(x)
-        if (length(metadata(x)) > 0)
-            metadata(x_ranges) <- metadata(x)
-        varnames <- colnames(x)
-
-        if (isSingleString(shift) && (shift %in% varnames))
-            shift <- values(x)[, shift]
-
-        if (isSingleString(weight) && (weight %in% varnames))
-            weight <- values(x)[, weight]
-
-        coverage(x_ranges, shift=shift, width=width, weight=weight,
-                           method=method)
+        .Defunct(msg=wmsg(.coverage_RangedData_defunct_msg))
     }
 )
 
