@@ -65,11 +65,13 @@ setGeneric("findOverlaps", signature=c("query", "subject"),
         standardGeneric("findOverlaps")
 )
 
-findOverlaps_Ranges <- function(query, subject,
+findOverlaps_IntegerRanges <- function(query, subject,
              maxgap=-1L, minoverlap=0L,
              type=c("any", "start", "end", "within", "equal"),
              select=c("all", "first", "last", "arbitrary"))
 {
+    if (is.integer(query))
+        query <- IRanges(query, width=1L)
     type <- match.arg(type)
     select <- match.arg(select)
     findOverlaps_NCList(query, subject,
@@ -77,7 +79,13 @@ findOverlaps_Ranges <- function(query, subject,
                         type=type, select=select)
 }
 
-setMethod("findOverlaps", c("Ranges", "Ranges"), findOverlaps_Ranges)
+setMethod("findOverlaps", c("IntegerRanges", "IntegerRanges"),
+    findOverlaps_IntegerRanges
+)
+
+setMethod("findOverlaps", c("integer", "IntegerRanges"),
+    findOverlaps_IntegerRanges
+)
 
 setMethod("findOverlaps", c("Vector", "missing"),
     function(query, subject, maxgap=-1L, minoverlap=0L,
@@ -95,52 +103,8 @@ setMethod("findOverlaps", c("Vector", "missing"),
     }
 )
 
-setMethod("findOverlaps", c("integer", "Ranges"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"))
-    {
-        findOverlaps(IRanges(query, query), subject,
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select))
-    }
-)
-
-setMethod("findOverlaps", c("Views", "Views"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"))
-    {
-        findOverlaps(ranges(query), ranges(subject),
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select))
-    }
-)
-
-setMethod("findOverlaps", c("Views", "Vector"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"))
-    {
-        findOverlaps(ranges(query), subject,
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select))
-    }
-)
-
-setMethod("findOverlaps", c("Vector", "Views"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"))
-    {
-        findOverlaps(query, ranges(subject),
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select))
-    }
-)
-
 # might consider making this the following:
-# setMethod("findOverlaps", c("RangesList", "RangesList"),
+# setMethod("findOverlaps", c("IntegerRangesList", "IntegerRangesList"),
 #           function(query, subject, maxgap = -1L, minoverlap = 0L,
 #                    type = c("any", "start", "end", "within", "equal"),
 #                    select = c("all", "first", "last", "arbitrary"),
@@ -152,7 +116,7 @@ setMethod("findOverlaps", c("Vector", "Views"),
 #           }
 #   )
 
-setMethod("findOverlaps", c("RangesList", "RangesList"),
+setMethod("findOverlaps", c("IntegerRangesList", "IntegerRangesList"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
                    select = c("all", "first", "last", "arbitrary"),
@@ -172,7 +136,7 @@ setMethod("findOverlaps", c("RangesList", "RangesList"),
             }
             ## NULL's are introduced where they do not match
             ## We replace those with empty IRanges
-            subject[sapply(subject, is.null)] <- list(IRanges())
+            subject[S4Vectors:::sapply_isNULL(subject)] <- list(IRanges())
 
             ans <- lapply(seq_len(length(subject)), function(i) {
               findOverlaps(query[[i]], subject[[i]],
@@ -198,45 +162,6 @@ setMethod("findOverlaps", c("RangesList", "RangesList"),
             ans
           })
 
-setMethod("findOverlaps", c("ViewsList", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"),
-             drop=FALSE)
-    {
-        findOverlaps(ranges(query), ranges(subject),
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select),
-                     drop=drop)
-    }
-)
-
-setMethod("findOverlaps", c("ViewsList", "Vector"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"),
-             drop=FALSE)
-    {
-        findOverlaps(ranges(query), subject,
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select),
-                     drop=drop)
-    }
-)
-
-setMethod("findOverlaps", c("Vector", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             select=c("all", "first", "last", "arbitrary"),
-             drop=FALSE)
-    {
-        findOverlaps(query, ranges(subject),
-                     maxgap=maxgap, minoverlap=minoverlap,
-                     type=match.arg(type), select=match.arg(select),
-                     drop=drop)
-    }
-)
-
 setMethod("findOverlaps", c("RangedData", "RangedData"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
@@ -249,7 +174,7 @@ setMethod("findOverlaps", c("RangedData", "RangedData"),
                          drop = drop)
           })
 
-setMethod("findOverlaps", c("RangedData", "RangesList"),
+setMethod("findOverlaps", c("RangedData", "IntegerRangesList"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
                    select = c("all", "first", "last", "arbitrary"),
@@ -261,7 +186,7 @@ setMethod("findOverlaps", c("RangedData", "RangesList"),
                          drop = drop)
           })
 
-setMethod("findOverlaps", c("RangesList", "RangedData"),
+setMethod("findOverlaps", c("IntegerRangesList", "RangedData"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
                    select = c("all", "first", "last", "arbitrary"),
@@ -322,6 +247,8 @@ setGeneric("countOverlaps", signature=c("query", "subject"),
              type=c("any", "start", "end", "within", "equal"),
              ...)
 {
+    if (is.integer(query))
+        query <- IRanges(query, width=1L)
     type <- match.arg(type)
     if (missing(subject)) {
         hits <- findOverlaps(query,
@@ -340,9 +267,10 @@ setGeneric("countOverlaps", signature=c("query", "subject"),
 }
 
 setMethod("countOverlaps", c("Vector", "Vector"), .countOverlaps_default)
+setMethod("countOverlaps", c("integer", "Vector"), .countOverlaps_default)
 setMethod("countOverlaps", c("Vector", "missing"), .countOverlaps_default)
 
-countOverlaps_Ranges <- function(query, subject,
+countOverlaps_IntegerRanges <- function(query, subject,
               maxgap=-1L, minoverlap=0L,
               type=c("any", "start", "end", "within", "equal"))
 {
@@ -354,9 +282,11 @@ countOverlaps_Ranges <- function(query, subject,
     ans
 }
 
-setMethod("countOverlaps", c("Ranges", "Ranges"), countOverlaps_Ranges)
+setMethod("countOverlaps", c("IntegerRanges", "IntegerRanges"),
+    countOverlaps_IntegerRanges
+)
 
-setMethod("countOverlaps", c("RangesList", "RangesList"),
+setMethod("countOverlaps", c("IntegerRangesList", "IntegerRangesList"),
           function(query, subject, maxgap=-1L, minoverlap=0L,
                    type = c("any", "start", "end", "within", "equal"))
           {
@@ -367,36 +297,6 @@ setMethod("countOverlaps", c("RangesList", "RangesList"),
                               SIMPLIFY = FALSE))
           })
 
-setMethod("countOverlaps", c("ViewsList", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"))
-    {
-         countOverlaps(ranges(query), ranges(subject),
-                       maxgap=maxgap, minoverlap=minoverlap,
-                       type=type)
-    }
-)
-
-setMethod("countOverlaps", c("ViewsList", "Vector"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"))
-    {
-         countOverlaps(ranges(query), subject,
-                       maxgap=maxgap, minoverlap=minoverlap,
-                       type=type)
-    }
-)
-
-setMethod("countOverlaps", c("Vector", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"))
-    {
-         countOverlaps(query, ranges(subject),
-                       maxgap=maxgap, minoverlap=minoverlap,
-                       type=type)
-    }
-)
-
 setMethod("countOverlaps", c("RangedData", "RangedData"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"))
@@ -404,14 +304,14 @@ setMethod("countOverlaps", c("RangedData", "RangedData"),
               countOverlaps(ranges(query), ranges(subject), maxgap = maxgap,
                             minoverlap = minoverlap, type = match.arg(type))
           })
-setMethod("countOverlaps", c("RangedData", "RangesList"),
+setMethod("countOverlaps", c("RangedData", "IntegerRangesList"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"))
           {
               countOverlaps(ranges(query), subject, maxgap = maxgap,
                             minoverlap = minoverlap, type = match.arg(type))
           })
-setMethod("countOverlaps", c("RangesList", "RangedData"),
+setMethod("countOverlaps", c("IntegerRangesList", "RangedData"),
           function(query, subject, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"))
           {
@@ -437,6 +337,8 @@ setGeneric("overlapsAny", signature=c("query", "subject"),
              type=c("any", "start", "end", "within", "equal"),
              ...)
 {
+    if (is.integer(query))
+        query <- IRanges(query, width=1L)
     type <- match.arg(type)
     if (missing(subject)) {
         ahit <- findOverlaps(query,
@@ -453,9 +355,10 @@ setGeneric("overlapsAny", signature=c("query", "subject"),
 }
 
 setMethod("overlapsAny", c("Vector", "Vector"), .overlapsAny_default)
+setMethod("overlapsAny", c("integer", "Vector"), .overlapsAny_default)
 setMethod("overlapsAny", c("Vector", "missing"), .overlapsAny_default)
 
-setMethod("overlapsAny", c("RangesList", "RangesList"),
+setMethod("overlapsAny", c("IntegerRangesList", "IntegerRangesList"),
     function(query, subject, maxgap=-1L, minoverlap=0L,
              type=c("any", "start", "end", "within", "equal"),
              ...)
@@ -471,7 +374,7 @@ setMethod("overlapsAny", c("RangesList", "RangesList"),
         }
         ## NULL's are introduced where they do not match
         ## We replace those with empty IRanges
-        subject[sapply(subject, is.null)] <- list(IRanges())
+        subject[S4Vectors:::sapply_isNULL(subject)] <- list(IRanges())
         LogicalList(lapply(structure(seq_len(length(query)),
                                      names = names(query)),
                            function(i)
@@ -482,44 +385,8 @@ setMethod("overlapsAny", c("RangesList", "RangesList"),
     }
 )
 
-setMethod("overlapsAny", c("ViewsList", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             ...)
-    {
-        overlapsAny(ranges(query), ranges(subject),
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
-    }
-)
-
-setMethod("overlapsAny", c("ViewsList", "Vector"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             ...)
-    {
-        overlapsAny(ranges(query), subject,
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
-    }
-)
-
-setMethod("overlapsAny", c("Vector", "ViewsList"),
-    function(query, subject, maxgap=-1L, minoverlap=0L,
-             type=c("any", "start", "end", "within", "equal"),
-             ...)
-    {
-        overlapsAny(query, ranges(subject),
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
-    }
-)
-
-.overlapsAny_RangedData_deprecation_msg <- c(
-    "The \"overlapsAny\" methods for RangedData objects are deprecated ",
+.overlapsAny_RangedData_defunct_msg <- c(
+    "The \"overlapsAny\" methods for RangedData objects are defunct ",
     "and won't be replaced. Please migrate your code to use GRanges or ",
     "GRangesList objects instead. RangedData objects will be deprecated ",
     "soon (their use has been discouraged since BioC 2.12, that is, since ",
@@ -531,37 +398,25 @@ setMethod("overlapsAny", c("RangedData", "RangedData"),
              type=c("any", "start", "end", "within", "equal"),
              ...)
     {
-        .Deprecated(msg=wmsg(.overlapsAny_RangedData_deprecation_msg))
-        overlapsAny(ranges(query), ranges(subject),
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
+        .Defunct(msg=wmsg(.overlapsAny_RangedData_defunct_msg))
     }
 )
 
-setMethod("overlapsAny", c("RangedData", "RangesList"),
+setMethod("overlapsAny", c("RangedData", "IntegerRangesList"),
     function(query, subject, maxgap=-1L, minoverlap=0L,
              type=c("any", "start", "end", "within", "equal"),
              ...)
     {
-        .Deprecated(msg=wmsg(.overlapsAny_RangedData_deprecation_msg))
-        overlapsAny(ranges(query), subject,
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
+        .Defunct(msg=wmsg(.overlapsAny_RangedData_defunct_msg))
     }
 )
 
-setMethod("overlapsAny", c("RangesList", "RangedData"),
+setMethod("overlapsAny", c("IntegerRangesList", "RangedData"),
     function(query, subject, maxgap=-1L, minoverlap=0L,
              type=c("any", "start", "end", "within", "equal"),
              ...)
     {
-        .Deprecated(msg=.wmsg(overlapsAny_RangedData_deprecation_msg))
-        overlapsAny(query, ranges(subject),
-                    maxgap=maxgap, minoverlap=minoverlap,
-                    type=match.arg(type),
-                    ...)
+        .Defunct(msg=wmsg(.overlapsAny_RangedData_defunct_msg))
     }
 )
 
@@ -602,8 +457,8 @@ setMethod("subsetByOverlaps", c("Vector", "Vector"),
     }
 )
 
-.subsetByOverlaps_RangedData_deprecation_msg <- c(
-    "The \"subsetByOverlaps\" methods for RangedData objects are deprecated ",
+.subsetByOverlaps_RangedData_defunct_msg <- c(
+    "The \"subsetByOverlaps\" methods for RangedData objects are defunct ",
     "and won't be replaced. Please migrate your code to use GRanges or ",
     "GRangesList objects instead. RangedData objects will be deprecated ",
     "soon (their use has been discouraged since BioC 2.12, that is, since ",
@@ -615,49 +470,23 @@ setMethod("subsetByOverlaps", c("RangedData", "RangedData"),
                    type = c("any", "start", "end", "within", "equal"),
                    invert = FALSE)
           {
-            .Deprecated(msg=wmsg(.subsetByOverlaps_RangedData_deprecation_msg))
-            ov_any <- unlist(!is.na(findOverlaps(ranges(x), ranges(ranges),
-                                                 maxgap = maxgap,
-                                                 minoverlap = minoverlap,
-                                                 type = match.arg(type),
-                                                 select = "arbitrary")),
-                             use.names=FALSE)
-            if (invert)
-                ov_any <- !ov_any
-            x[ov_any]
+            .Defunct(msg=wmsg(.subsetByOverlaps_RangedData_defunct_msg))
           })
 
-setMethod("subsetByOverlaps", c("RangedData", "RangesList"),
+setMethod("subsetByOverlaps", c("RangedData", "IntegerRangesList"),
           function(x, ranges, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
                    invert = FALSE)
           {
-            .Deprecated(msg=wmsg(.subsetByOverlaps_RangedData_deprecation_msg))
-            ov_any <- unlist(!is.na(findOverlaps(ranges(x), ranges,
-                                                 maxgap = maxgap,
-                                                 minoverlap = minoverlap,
-                                                 type = match.arg(type),
-                                                 select = "arbitrary")),
-                             use.names=FALSE)
-            if (invert)
-                ov_any <- !ov_any
-            x[ov_any]
+            .Defunct(msg=wmsg(.subsetByOverlaps_RangedData_defunct_msg))
           })
 
-setMethod("subsetByOverlaps", c("RangesList", "RangedData"),
+setMethod("subsetByOverlaps", c("IntegerRangesList", "RangedData"),
           function(x, ranges, maxgap = -1L, minoverlap = 0L,
                    type = c("any", "start", "end", "within", "equal"),
                    invert = FALSE)
           {
-            .Deprecated(msg=wmsg(.subsetByOverlaps_RangedData_deprecation_msg))
-            ov_any <- !is.na(findOverlaps(x, ranges(ranges),
-                                          maxgap = maxgap,
-                                          minoverlap = minoverlap,
-                                          type = match.arg(type),
-                                          select = "arbitrary"))
-            if (invert)
-                ov_any <- !ov_any
-            x[ov_any]
+            .Defunct(msg=wmsg(.subsetByOverlaps_RangedData_defunct_msg))
           })
 
 
@@ -671,7 +500,7 @@ setGeneric("overlapsRanges", signature=c("query", "subject"),
     function(query, subject, hits=NULL, ...) standardGeneric("overlapsRanges")
 )
 
-setMethod("overlapsRanges", c("Ranges", "Ranges"),
+setMethod("overlapsRanges", c("IntegerRanges", "IntegerRanges"),
     function(query, subject, hits=NULL, ...)
     {
         if (is.null(hits)) {
@@ -690,8 +519,8 @@ setMethod("overlapsRanges", c("Ranges", "Ranges"),
         }
         ### Could be replaced by 1-liner:
         ###   pintersect(query[queryHits(hits)], subject[subjectHits(hits)])
-        ### but will fail if 'query' or 'subject' is a kind of Ranges object
-        ### that cannot be subsetted (e.g. Partitioning object).
+        ### but will fail if 'query' or 'subject' is a kind of IntegerRanges
+        ### object that cannot be subsetted (e.g. Partitioning object).
         m <- as.matrix(hits)
         qstart <- start(query)[m[,1L]]
         qend <- end(query)[m[,1L]]
@@ -701,7 +530,7 @@ setMethod("overlapsRanges", c("Ranges", "Ranges"),
     }
 )
 
-setMethod("overlapsRanges", c("RangesList", "RangesList"),
+setMethod("overlapsRanges", c("IntegerRangesList", "IntegerRangesList"),
     function(query, subject, hits=NULL, ...)
     {
         if (is.null(hits)) {
@@ -721,9 +550,9 @@ setMethod("overlapsRanges", c("RangesList", "RangesList"),
         queries <- as.list(query, use.names = FALSE)
         subjects <- as.list(subject, use.names = FALSE)
         els <- as.list(hits, use.names = FALSE)
-        ans <- do.call(RangesList, lapply(seq_len(length(hits)), function(i) {
-          overlapsRanges(queries[[i]], subjects[[i]], els[[i]])
-        }))
+        ans <- lapply(seq_len(length(hits)),
+            function(i) overlapsRanges(queries[[i]], subjects[[i]], els[[i]]))
+        ans <- as(ans, "SimpleIRangesList")
         names(ans) <- names(hits)
         ans
     }
@@ -732,31 +561,17 @@ setMethod("overlapsRanges", c("RangesList", "RangesList"),
 setMethod("ranges", "Hits", function(x, use.names=TRUE, use.mcols=FALSE,
                                         query, subject)
 {
-    msg <- c("\"ranges\" method for Hits objects is deprecated. ",
+    msg <- c("\"ranges\" method for Hits objects is defunct. ",
              "Please use overlapsRanges() instead.")
-    .Deprecated(msg=wmsg(msg))
-    query_is_missing <- missing(query)
-    subject_is_missing <- missing(subject)
-    if (query_is_missing || subject_is_missing) {
-        query <- if (subject_is_missing) use.names else use.mcols
-        subject <- if (query_is_missing) use.mcols else use.names
-    }
-    overlapsRanges(query, subject, x)
+    .Defunct(msg=wmsg(msg))
 })
 
 setMethod("ranges", "HitsList", function(x, use.names=TRUE, use.mcols=FALSE,
                                             query, subject)
 {
-    msg <- c("\"ranges\" method for HitsList objects is deprecated. ",
+    msg <- c("\"ranges\" method for HitsList objects is defunct. ",
              "Please use overlapsRanges() instead.")
-    .Deprecated(msg=wmsg(msg))
-    query_is_missing <- missing(query)
-    subject_is_missing <- missing(subject)
-    if (query_is_missing || subject_is_missing) {
-        query <- if (subject_is_missing) use.names else use.mcols
-        subject <- if (query_is_missing) use.mcols else use.names
-    }
-    overlapsRanges(query, subject, x)
+    .Defunct(msg=wmsg(msg))
 })
 
 
@@ -771,7 +586,7 @@ setGeneric("poverlaps", signature=c("query", "subject"),
                standardGeneric("poverlaps")
            )
 
-setMethod("poverlaps", c("Ranges", "Ranges"),
+setMethod("poverlaps", c("IntegerRanges", "IntegerRanges"),
           function(query, subject, maxgap=0L, minoverlap=1L,
                    type=c("any", "start", "end", "within", "equal"))
           {
@@ -804,7 +619,7 @@ setMethod("poverlaps", c("Ranges", "Ranges"),
           }
           )
 
-setMethod("poverlaps", c("integer", "Ranges"),
+setMethod("poverlaps", c("integer", "IntegerRanges"),
           function(query, subject, maxgap=0L, minoverlap=1L,
                    type=c("any", "start", "end", "within", "equal"))
           {
@@ -813,7 +628,7 @@ setMethod("poverlaps", c("integer", "Ranges"),
                         type=match.arg(type))
           })
 
-setMethod("poverlaps", c("Ranges", "integer"),
+setMethod("poverlaps", c("IntegerRanges", "integer"),
           function(query, subject, maxgap=0L, minoverlap=1L,
                    type=c("any", "start", "end", "within", "equal"))
           {
