@@ -1,39 +1,88 @@
 ### =========================================================================
 ### Intra-range methods
 ### -------------------------------------------------------------------------
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### update_ranges()
 ###
 
+### For internal use by the intra-range methods.
+### Update the ranges of a Ranges derivative. Return an object of the same
+### class as and parallel to 'x'.
+setGeneric("update_ranges", signature="x",
+    function(x, start=NULL, end=NULL, width=NULL, use.names=TRUE)
+        standardGeneric("update_ranges")
+)
 
-normargAtomicList1 <- function(arg, List, lx, argname = deparse(substitute(arg)))
+### Shallow check: only check type and length, not the content.
+.check_start_or_end_or_width <- function(start, x_len)
 {
-    if (is.vector(arg))
-        arg <- List(as.list(S4Vectors:::recycleVector(arg, lx)))
-    else if (!is(arg, "AtomicList"))
-        stop("'", argname,"' must be a vector or AtomicList object")
-    arg
+    if (is.null(start))
+        return()
+    stopifnot(is.integer(start))
+    stopifnot(length(start) == x_len)
 }
 
-normargAtomicList2 <- function(arg, List, lx, x_eltNROWS,
-                               argname = deparse(substitute(arg)))
-{
-    if (!(is.vector(arg) && length(arg) == 1L)) {
-        if (is.vector(arg))
-          arg <- as(rep(S4Vectors:::recycleVector(arg, lx), x_eltNROWS),
-                    class(unlist(List())))
-        else {
-          if (!is(arg, "AtomicList"))
-            stop("'arg' must be a vector or AtomicList object")
-          if (!isTRUE(all.equal(elementNROWS(arg), x_eltNROWS,
-                                check.attributes=FALSE)))
-            arg <- mapply(S4Vectors:::recycleVector,
-                          arg, List(as.list(x_eltNROWS)))
-          arg <- unlist(arg, use.names=FALSE)
+### Does not validate the modified object.
+setMethod("update_ranges", "IRanges",
+    function(x, start=NULL, end=NULL, width=NULL, use.names=TRUE)
+    {
+        use.names <- S4Vectors:::normargUseNames(use.names)
+        narg <- sum(!is.null(start), !is.null(end), !is.null(width))
+        if (narg == 0L) {
+            if (!(use.names || is.null(x@NAMES)))
+                x@NAMES <- NULL
+            return(x)
         }
-    } else if (is.list(arg)){
-        arg <- unlist(arg, use.names=FALSE)
+        stopifnot(narg <= 2L)
+        x_len <- length(x)
+        .check_start_or_end_or_width(start, x_len)
+        .check_start_or_end_or_width(end, x_len)
+        .check_start_or_end_or_width(width, x_len)
+        if (narg == 2L) {
+            if (!is.null(end)) {
+                if (is.null(start)) {
+                    ## 'end' and 'width' supplied
+                    start <- end - width + 1L
+                } else {
+                    ## 'start' and 'end' supplied
+                    width <- end - start + 1L
+                }
+            }
+            args <- list(start=start, width=width)
+        } else {
+            stopifnot(is.null(width))
+            if (is.null(start)) {
+                ## only 'end' supplied
+                width <- end - x@start + 1L
+                args <- list(width=width)
+            } else {
+                ## only 'start' supplied
+                width <- x@width - (start - x@start)
+                args <- list(start=start, width=width)
+            }
+        }
+        if (use.names) {
+            more_args <- list(check=FALSE)
+        } else {
+            more_args <- list(NAMES=NULL, check=FALSE)
+        }
+        args <- c(list(x), args, more_args)
+        do.call(BiocGenerics:::replaceSlots, args)
     }
-    arg
-}
+)
+
+setMethod("update_ranges", "Views",
+    function(x, start=NULL, end=NULL, width=NULL, use.names=TRUE)
+    {
+        x@ranges <- update_ranges(x@ranges, start=start,
+                                            end=end,
+                                            width=width,
+                                            use.names=use.names)
+        x
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -52,13 +101,9 @@ setMethod("shift", "Ranges",
                  "a NormalIRanges object")
         shift <- recycleIntegerArg(shift, "shift", length(x))
         new_start <- start(x) + shift
-        if (is(x, "IRanges")) {
-            x@start <- new_start
-        } else {
-            x <- update(x, start=new_start, width=width(x), check=FALSE)
-        }
-        if (!S4Vectors:::normargUseNames(use.names))
-            names(x) <- NULL
+        x <- update_ranges(x, start=new_start,
+                              width=width(x),
+                              use.names=use.names)
         validObject(x)
         x
     }
@@ -85,98 +130,51 @@ setMethod("shift", "IPos",
                 shift <- shift[[1L]]
             }
         }
-        x@pos_runs <- callGeneric(x@pos_runs, shift=shift)
-        x
-    }
-)
-
-setMethod("shift", "Views",
-    function(x, shift=0L, use.names=TRUE)
-    {
-        x@ranges <- shift(ranges(x), shift=shift, use.names=use.names)
-        x
+        new_pos_runs <- callGeneric(x@pos_runs, shift=shift)
+        BiocGenerics:::replaceSlots(x, pos_runs=new_pos_runs, check=FALSE)
     }
 )
 
 setMethod("shift", "RangesList",
-          function(x, shift=0L, use.names = TRUE)
-          {
-              lx <- length(x)
-              shift <- normargAtomicList1(shift, IntegerList, lx)
-              mendoapply("shift", x = x, shift = shift,
-                         MoreArgs = list(use.names = use.names))
-          })
+    function(x, shift=0L, use.names=TRUE)
+    {
+        if (!is(shift, "List"))
+            shift <- as(shift, "List")
+        shift <- S4Vectors:::VH_recycle(shift, x, "shift", "x")
 
-setMethod("shift", "CompressedIRangesList",
-          function(x, shift=0L, use.names = TRUE)
-          {
-              lx <- length(x)
-              x_eltNROWS <- elementNROWS(x)
-              shift <- normargAtomicList2(shift, IntegerList, lx, x_eltNROWS)
-              slot(x, "unlistData", check=FALSE) <-
-                shift(x@unlistData, shift = shift, use.names = use.names)
-              x
-          })
+        if (is(x, "CompressedRangesList")) {
+            unlisted_shift <- unlist(shift, use.names=FALSE)
+            new_unlistData <- shift(x@unlistData, shift=unlisted_shift,
+                                                  use.names=use.names)
+            ans <- BiocGenerics:::replaceSlots(x, unlistData=new_unlistData,
+                                                  check=FALSE)
+            return(ans)
+        }
+        mendoapply(shift, x, shift,
+                          MoreArgs=list(use.names=use.names))
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### narrow()
 ###
 
-setGeneric("narrow", signature="x",
-    function(x, start=NA, end=NA, width=NA, use.names=TRUE)
-        standardGeneric("narrow")
-)
-
-setMethod("narrow", "Ranges",
-    function(x, start=NA, end=NA, width=NA, use.names=TRUE)
+### The default "narrow" method calls windows() so we only need to implement
+### a "windows" method for IntegerRanges objects to make narrow() work on
+### these objects.
+setMethod("windows", "Ranges",
+    function(x, start=NA, end=NA, width=NA)
     {
-        if (is(x, "NormalIRanges"))
-            stop("narrowing a NormalIRanges object is not supported")
-        solved_SEW <- solveUserSEW(width(x), start=start, end=end, width=width)
-        ans_start <- start(x) + start(solved_SEW) - 1L
-        ans_width <- width(solved_SEW)
-        x <- update(x, start=ans_start, width=ans_width, check=FALSE)
-        if (!S4Vectors:::normargUseNames(use.names))
-            names(x) <- NULL
-        x
+        ir <- make_IRanges_from_windows_args(x, start, end, width)
+        if (length(x) == 0L)
+            return(x)
+        ans_start <- start(x) + start(ir) - 1L
+        ans_width <- width(ir)
+        update_ranges(x, start=ans_start, width=ans_width)
     }
 )
 
-setMethod("narrow", "Views",
-    function(x, start=NA, end=NA, width=NA, use.names=TRUE)
-    {
-        x@ranges <- narrow(ranges(x), start=start, end=end, width=width,
-                           use.names=use.names)
-        x
-    }
-)
-
-setMethod("narrow", "RangesList",
-          function(x, start = NA, end = NA, width = NA, use.names = TRUE)
-          {
-              lx <- length(x)
-              start <- normargAtomicList1(start, IntegerList, lx)
-              end <- normargAtomicList1(end, IntegerList, lx)
-              width <- normargAtomicList1(width, IntegerList, lx)
-              mendoapply(narrow, x = x, start = start, end = end, width = width,
-                         MoreArgs = list(use.names = use.names))
-          })
-
-setMethod("narrow", "CompressedIRangesList",
-          function(x, start = NA, end = NA, width = NA, use.names = TRUE)
-          {
-              lx <- length(x)
-              x_eltNROWS <- elementNROWS(x)
-              start <- normargAtomicList2(start, IntegerList, lx, x_eltNROWS)
-              end <- normargAtomicList2(end, IntegerList, lx, x_eltNROWS)
-              width <- normargAtomicList2(width, IntegerList, lx, x_eltNROWS)
-              slot(x, "unlistData", check=FALSE) <-
-                narrow(x@unlistData, start = start, end = end, width = width,
-                       use.names = use.names)
-              x
-          })
-
 setMethod("narrow", "MaskCollection",
     function(x, start=NA, end=NA, width=NA, use.names=TRUE)
     {
@@ -218,7 +216,7 @@ setMethod("resize", "Ranges",
         if (S4Vectors:::anyMissingOrOutside(width, 0L))
             stop("'width' values must be non-negative")
         if (!(is.character(fix) ||
-              (is(fix, "Rle") && is.character(runValue(fix)))) || 
+              (is(fix, "Rle") && is.character(runValue(fix)))) ||
             (length(fix) == 0L && length(x) > 0L) ||
             (length(setdiff(unique(fix),
                             c("start", "end", "center"))) > 0)) {
@@ -247,35 +245,35 @@ setMethod("resize", "Ranges",
                 ans_start <- replaceROWS(ans_start, fixCenter, value)
             }
         }
-        x <- update(x, start=ans_start, width=ans_width, check=FALSE)
-        if (!S4Vectors:::normargUseNames(use.names))
-            names(x) <- NULL
-        x
+        update_ranges(x, start=ans_start, width=ans_width, use.names=use.names)
     }
 )
 
 setMethod("resize", "RangesList",
-          function(x, width, fix = "start", use.names = TRUE)
-          {
-              lx <- length(x)
-              width <- normargAtomicList1(width, IntegerList, lx)
-              fix <- normargAtomicList1(fix, CharacterList, lx)
-              mendoapply(resize, x = x, width = width, fix = fix,
-                         MoreArgs = list(use.names = use.names))
-          })
+    function(x, width, fix="start", use.names=TRUE, ...)
+    {
+        if (!is(width, "List"))
+            width <- as(width, "List")
+        width <- S4Vectors:::VH_recycle(width, x, "width", "x")
+        if (!is(fix, "List"))
+            fix <- as(fix, "List")
+        fix <- S4Vectors:::VH_recycle(fix, x, "fix", "x")
 
-setMethod("resize", "CompressedIRangesList",
-          function(x, width, fix = "start", use.names = TRUE)
-          {
-              lx <- length(x)
-              x_eltNROWS <- elementNROWS(x)
-              width <- normargAtomicList2(width, IntegerList, lx, x_eltNROWS)
-              fix <- normargAtomicList2(fix, CharacterList, lx, x_eltNROWS)
-              slot(x, "unlistData", check=FALSE) <-
-                resize(x@unlistData, width = width, fix = fix,
-                       use.names = use.names)
-              x
-          })
+        if (is(x, "CompressedRangesList")) {
+            unlisted_width <- unlist(width, use.names=FALSE)
+            unlisted_fix <- unlist(fix, use.names=FALSE)
+            new_unlistData <- resize(x@unlistData, width=unlisted_width,
+                                                   fix=unlisted_fix,
+                                                   use.names=use.names,
+                                                   ...)
+            ans <- BiocGenerics:::replaceSlots(x, unlistData=new_unlistData,
+                                                  check=FALSE)
+            return(ans)
+        }
+        mendoapply(resize, x, width, fix,
+                           MoreArgs=list(use.names=use.names, ...))
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -317,92 +315,90 @@ setMethod("flank", "Ranges",
             ans_start[idx2a] <- end(x)[idx2a] + 1L
             ans_start[idx2b] <- end(x)[idx2b] + width[idx2b] + 1L
         }
-        x <- update(x, start=ans_start, width=ans_width, check=FALSE)
-        if (!S4Vectors:::normargUseNames(use.names))
-            names(x) <- NULL
-        x
+        update_ranges(x, start=ans_start, width=ans_width, use.names=use.names)
     }
 )
 
 setMethod("flank", "RangesList",
-          function(x, width, start = TRUE, both = FALSE, use.names = TRUE)
-          {
-              lx <- length(x)
-              width <- normargAtomicList1(width, IntegerList, lx)
-              start <- normargAtomicList1(start, LogicalList, lx)
-              mendoapply(flank, x = x, width = width, start = start,
-                         MoreArgs = list(both = both, use.names = use.names))
-          })
-
-setMethod("flank", "CompressedIRangesList",
-          function(x, width, start = TRUE, both = FALSE, use.names = TRUE)
-          {
-              lx <- length(x)
-              x_eltNROWS <- elementNROWS(x)
-              width <- normargAtomicList2(width, IntegerList, lx, x_eltNROWS)
-              start <- normargAtomicList2(start, LogicalList, lx, x_eltNROWS)
-              slot(x, "unlistData", check=FALSE) <-
-                flank(x@unlistData, width = width, start = start, both = both,
-                      use.names = use.names)
-              x
-          })
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### promoters() 
-###
-
-setGeneric("promoters", signature="x",
-    function(x, upstream=2000, downstream=200, ...)
-        standardGeneric("promoters")
-)
-
-setMethod("promoters", "Ranges",
-    function(x, upstream=2000, downstream=200, ...)
+    function(x, width, start=TRUE, both=FALSE, use.names=TRUE, ...)
     {
-        if (is(x, "NormalIRanges"))
-            stop("promoters on a NormalIRanges object is not supported")
-        if (!isSingleNumber(upstream))
-            stop("'upstream' must be a single integer")
-        if (!is.integer(upstream))
-            upstream <- as.numeric(upstream)
-        if (!isSingleNumber(downstream))
-            stop("'downstream' must be a single integer")
-        if (!is.integer(downstream))
-            downstream <- as.numeric(downstream)
-        if (upstream < 0 | downstream < 0)
-            stop("'upstream' and 'downstream' must be integers >= 0")
-        st <- start(x)
-        start(x) <- st - upstream
-        end(x) <- st + downstream - 1L
-        x
+        if (!is(width, "List"))
+            width <- as(width, "List")
+        width <- S4Vectors:::VH_recycle(width, x, "width", "x")
+        if (!is(start, "List"))
+            start <- as(start, "List")
+        start <- S4Vectors:::VH_recycle(start, x, "start", "x")
+
+        if (is(x, "CompressedRangesList")) {
+            unlisted_width <- unlist(width, use.names=FALSE)
+            unlisted_start <- unlist(start, use.names=FALSE)
+            new_unlistData <- flank(x@unlistData, width=unlisted_width,
+                                                  start=unlisted_start,
+                                                  both=both,
+                                                  use.names=use.names,
+                                                  ...)
+            ans <- BiocGenerics:::replaceSlots(x, unlistData=new_unlistData,
+                                                  check=FALSE)
+            return(ans)
+        }
+        mendoapply(flank, x, width, start,
+                          MoreArgs=list(both=both, use.names=use.names, ...))
     }
 )
 
-setMethod("promoters", "Views",
-    function(x, upstream=2000, downstream=200, ...)
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### promoters()
+###
+
+setGeneric("promoters", signature="x",
+    function(x, upstream=2000, downstream=200, use.names=TRUE, ...)
+        standardGeneric("promoters")
+)
+
+setMethod("promoters", "IntegerRanges",
+    function(x, upstream=2000, downstream=200, use.names=TRUE)
     {
-        x@ranges <- promoters(ranges(x), upstream, downstream)
-        x
+        if (is(x, "NormalIRanges"))
+            stop("promoters() is not supported on a NormalIRanges object")
+        x_len <- length(x)
+        upstream <- recycleIntegerArg(upstream, "upstream", x_len)
+        downstream <- recycleIntegerArg(downstream, "downstream", x_len)
+        if (x_len == 0L)
+            return(update_ranges(x, use.names=use.names))
+        if (min(upstream) < 0L || min(downstream) < 0L)
+            stop("'upstream' and 'downstream' must be integers >= 0")
+        x_start <- start(x)
+        new_start <- x_start - upstream
+        new_end <- x_start + downstream - 1L
+        update_ranges(x, start=new_start, end=new_end, use.names=use.names)
     }
 )
 
 setMethod("promoters", "RangesList",
-          function(x, upstream=2000, downstream=200, ...) 
-          {
-              endoapply(x, promoters, upstream = upstream, 
-                         downstream = downstream)
-          }
-)
+    function(x, upstream=2000, downstream=200, use.names=TRUE)
+    {
+        if (!is(upstream, "List"))
+            upstream <- as(upstream, "List")
+        upstream <- S4Vectors:::VH_recycle(upstream, x, "upstream", "x")
+        if (!is(downstream, "List"))
+            downstream <- as(downstream, "List")
+        downstream <- S4Vectors:::VH_recycle(downstream, x, "downstream", "x")
 
-setMethod("promoters", "CompressedIRangesList",
-          function(x, upstream=2000, downstream=200, ...)
-          {
-              slot(x, "unlistData", check=FALSE) <-
-                promoters(x@unlistData, upstream = upstream, 
-                          downstream = downstream, ...)
-              x
-          }
+        if (is(x, "CompressedRangesList")) {
+            unlisted_upstream <- unlist(upstream, use.names=FALSE)
+            unlisted_downstream <- unlist(downstream, use.names=FALSE)
+            new_unlistData <- promoters(x@unlistData,
+                                        upstream=unlisted_upstream,
+                                        downstream=unlisted_downstream,
+                                        use.names=use.names)
+            ans <- BiocGenerics:::replaceSlots(x, unlistData=new_unlistData,
+                                                  check=FALSE)
+            return(ans)
+        }
+        mendoapply(promoters, x, upstream, downstream,
+                              MoreArgs=list(use.names=use.names))
+    }
 )
 
 
@@ -415,22 +411,19 @@ setGeneric("reflect", signature="x",
         standardGeneric("reflect")
 )
 
-setMethod("reflect", "Ranges",
+setMethod("reflect", "IntegerRanges",
     function(x, bounds, use.names=TRUE)
     {
         if (is(x, "NormalIRanges"))
             stop("reflecting a NormalIRanges object is not supported")
-        if (!is(bounds, "Ranges"))
-            stop("'bounds' must be a Ranges object")
+        if (!is(bounds, "IntegerRanges"))
+            stop("'bounds' must be an IntegerRanges object")
         if (length(x) > 1 && length(bounds) == 0)
-            stop("'bounds' is an empty Ranges object")
+            stop("'bounds' is an empty IntegerRanges object")
         if (length(x) < length(bounds))
             bounds <- head(bounds, length(x))
         ans_start <- (2L * start(bounds) + width(bounds) - 1L) - end(x)
-        x <- update(x, start=ans_start, width=width(x), check=FALSE)
-        if (!S4Vectors:::normargUseNames(use.names))
-            names(x) <- NULL
-        x
+        update_ranges(x, start=ans_start, width=width(x), use.names=use.names)
     }
 )
 
@@ -463,9 +456,8 @@ setGeneric("restrict", signature="x",
 ###       Those that are not overlapping but are however adjacent are kept
 ###       (and are empty after restriction).
 ###   2L: All ranges in 'x' are kept after restriction.
-###       Ranges
 ### Note that the only mode compatible with a NormalIRanges object is 0L.
-Ranges.restrict <- function(x, start, end, drop.ranges.mode, use.names)
+.restrict_IntegerRanges <- function(x, start, end, drop.ranges.mode, use.names)
 {
     if (!S4Vectors:::isNumericOrNAs(start))
         stop("'start' must be a vector of integers")
@@ -537,14 +529,14 @@ Ranges.restrict <- function(x, start, end, drop.ranges.mode, use.names)
     ans_end[too_right] <- end[too_right]
 
     ans_width <- ans_end - ans_start + 1L
-    update(x, start=ans_start,
-              width=ans_width,
-              names=ans_names,
-              mcols=ans_mcols,
-              check=FALSE)
+    BiocGenerics:::replaceSlots(x, start=ans_start,
+                                   width=ans_width,
+                                   NAMES=ans_names,
+                                   mcols=ans_mcols,
+                                   check=FALSE)
 }
 
-setMethod("restrict", "Ranges",
+setMethod("restrict", "IntegerRanges",
     function(x, start=NA, end=NA, keep.all.ranges=FALSE, use.names=TRUE)
     {
         if (!isTRUEorFALSE(keep.all.ranges))
@@ -561,42 +553,48 @@ setMethod("restrict", "Ranges",
             else
                 drop.ranges.mode <- 1L
         }
-        Ranges.restrict(x, start, end, drop.ranges.mode, use.names)
+        .restrict_IntegerRanges(x, start, end, drop.ranges.mode, use.names)
+    }
+)
+
+setMethod("restrict", "Views",
+    function(x, start=NA, end=NA, keep.all.ranges=FALSE, use.names=TRUE)
+    {
+        new_ranges <- restrict(ranges(x), start=start, end=end,
+                                          keep.all.ranges=keep.all.ranges,
+                                          use.names=use.names)
+        BiocGenerics:::replaceSlots(x, ranges=new_ranges, check=FALSE)
     }
 )
 
 setMethod("restrict", "RangesList",
-          function(x, start = NA, end = NA, keep.all.ranges = FALSE,
-                   use.names = TRUE)
-          {
-              lx <- length(x)
-              start <- normargAtomicList1(start, IntegerList, lx)
-              end <- normargAtomicList1(end, IntegerList, lx)
-              mendoapply(restrict, x, start = start, end = end,
-                         MoreArgs = list(keep.all.ranges = keep.all.ranges,
-                           use.names = use.names))
-          })
+    function(x, start=NA, end=NA, keep.all.ranges=FALSE, use.names=TRUE)
+    {
+        if (!isTRUEorFALSE(keep.all.ranges))
+            stop("'keep.all.ranges' must be TRUE or FALSE")
+        if (!is(start, "List"))
+            start <- as(start, "List")
+        start <- S4Vectors:::VH_recycle(start, x, "start", "x")
+        if (!is(end, "List"))
+            end <- as(end, "List")
+        end <- S4Vectors:::VH_recycle(end, x, "end", "x")
 
-setMethod("restrict", "CompressedIRangesList",
-          function(x, start = NA, end = NA, keep.all.ranges = FALSE,
-                   use.names = TRUE)
-          {
-              if (!isTRUEorFALSE(keep.all.ranges))
-                  stop("'keep.all.ranges' must be TRUE or FALSE")
-              if (keep.all.ranges) {
-                  lx <- length(x)
-                  x_eltNROWS <- elementNROWS(x)
-                  start <- normargAtomicList2(start, IntegerList,
-                                              lx, x_eltNROWS)
-                  end <- normargAtomicList2(end, IntegerList, lx, x_eltNROWS)
-                  slot(x, "unlistData", check=FALSE) <-
-                    restrict(x@unlistData, start = start, end = end,
-                             keep.all.ranges = keep.all.ranges,
-                             use.names = use.names)
-              } else
-                  x <- callNextMethod()
-              x
-          })
+        if (is(x, "CompressedRangesList") && keep.all.ranges) {
+            unlisted_start <- unlist(start, use.names=FALSE)
+            unlisted_end <- unlist(end, use.names=FALSE)
+            new_unlistData <- restrict(x@unlistData, start=unlisted_start,
+                                                     end=unlisted_end,
+                                                     keep.all.ranges=TRUE,
+                                                     use.names=use.names)
+            ans <- BiocGenerics:::replaceSlots(x, unlistData=new_unlistData,
+                                                  check=FALSE)
+            return(ans)
+        }
+        mendoapply(restrict, x, start, end,
+                             MoreArgs=list(keep.all.ranges=keep.all.ranges,
+                                           use.names=use.names))
+    }
+)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -607,6 +605,7 @@ setGeneric("threebands", signature="x",
     function(x, start=NA, end=NA, width=NA)
         standardGeneric("threebands")
 )
+
 ### Method for IRanges only!
 setMethod("threebands", "IRanges",
     function(x, start=NA, end=NA, width=NA)
@@ -645,7 +644,7 @@ setMethod("Ops", c("Ranges", "numeric"),
             r <- e1
             mid <- (start(r)+end(r))/2
             w <- width(r)/e2
-            update(r, start = ceiling(mid - w/2), width = floor(w), check = FALSE)
+            update_ranges(r, start = as.integer(ceiling(mid - w/2)), width = as.integer(floor(w)))
         } else {
             if (.Generic == "-") {
                 e2 <- -e2
@@ -654,7 +653,7 @@ setMethod("Ops", c("Ranges", "numeric"),
             if (.Generic == "+") {
                 if (any(-e2*2 > width(e1)))
                     stop("adjustment would result in ranges with negative widths")
-                update(e1, start = start(e1) - e2, end = end(e1) + e2, check = FALSE)
+                update_ranges(e1, start = as.integer(start(e1) - e2), end = as.integer(end(e1) + e2))
             }
         }
     }
@@ -668,7 +667,7 @@ setMethod("Ops", c("RangesList", "numeric"),
             e1
           })
 
-setMethod("Ops", c("CompressedIRangesList", "numeric"),
+setMethod("Ops", c("CompressedRangesList", "numeric"),
           function(e1, e2)
           {
             relist(callGeneric(unlist(e1, use.names = FALSE), e2), e1)
